#include <Fade_2D.h>
#include <Visualizer3.h>
#include "someTools.h"



using namespace std;
using namespace GEOM_FADE25D;

// Prototypes
int terrain_main(); // Main function
void terrain_full(); // 0...Triangulate using all points
void terrain_adaptiveSimplify(); // 1...Simplify adaptively according to height tolerance
void terrain_gridSimplify(); // 2...Simplify according to the grid size
void terrain_numPointsSimplify(); // 3...Simplify according to the desired number of points
void exportData(); // 4...Export a triangulation
void getInputPoints(const std::string& filename,std::vector<Point2>& vPointsOut); // Creates input
void write(const std::string& id,const std::string& name,const Fade_2D& dt); // Writer (*.list and *.obj)
void getMountain(std::vector<Point2>& vPointsOut); // Returns a test terrain



// * main() - control
int terrain_main()
{
	std::cout<<gRed("\n\n\n+++ +++ +++ +++ +++ +++ +++")<<std::endl;
	while(true)
	{
		std::cout<<"\n\n* Fade2.5D Demo - Terrain triangulation\n"<<std::endl;

		// * 1 *   Let the user choose
		cout << "0...Terrain - Full Triangulation" << endl;
		cout << "1...Terrain - Adaptive Simplification" << endl;
		cout << "2...Terrain - Grid Simplification" << endl;
		cout << "3...Terrain - NumPoints Simplification" << endl;
		cout << "4...Export - Triangulation Data Export" << endl;
		cout << "q...quit" << endl;
		cout << "\n\nChoose an example [0-4],q: ";
		char choice(0);
		cin>>choice;
		cout<<"------------------------------\n\n";
		switch(choice)
		{
			case '0':
				cout<<"* Terrain full"<<std::endl;
				terrain_full();
				break;
			case '1':
				cout<<"* Adaptive simplify"<<std::endl;
				terrain_adaptiveSimplify();
				break;
			case '2':
				cout<<"* Grid simplify"<<std::endl;
				terrain_gridSimplify();
				break;
			case '3':
				cout<<"* NumPoints simplify"<<std::endl;
				terrain_numPointsSimplify();
				break;
			case '4':
				cout<<"* Export a triangulation"<<std::endl;
				exportData();
				break;
			case 'q':
			default:
				cout<<"Returning"<<endl;
				return 1;
		}
		cout<<"\n\nEND\n------------------------------"<<endl;
	}
}


// 0: Triangulate all points
void terrain_full()
{
	// * 1 *   Get input points
	std::vector<Point2> vInputPoints;
	getInputPoints("MOUNTAIN",vInputPoints);

	// * 2 *   Triangulate
	Fade_2D dt;
	// dt.insert(vInputPoints); // Still possible but CloudPrepare is recommended
	CloudPrepare cloudPrep;
	cloudPrep.add(vInputPoints);
	dt.insert(&cloudPrep,true); // More memory efficient

	// * 3 *   Show
	write("a0","terrain_full",dt);
}


// 1: Simplify: Adaptive, cluster points based on their height difference
void terrain_adaptiveSimplify()
{
	// * 1 *   Get input points
	std::vector<Point2> vInputPoints;
	getInputPoints("MOUNTAIN",vInputPoints);

	// * 2 *   Add them to a CloudPrepare object (or add them one by one)
	CloudPrepare cloudPrep;
	cloudPrep.add(vInputPoints);

	// * 3 *   Reduce the cloud size
	double maxDiffZ(1.0); // z-tolerance per point group
	SumStrategy sms(SMS_AVERAGE); // or SMS_MEDIAN, SMS_MINIMUM, SMS_MAXIMUM
	ConvexHullStrategy chs(CHS_MAXHULL); // or CHS_NONE, CHS_MINHULL
	// We can use dry-run=true to determine the number of points that would result:
	double aTolerance []={0.1,0.5,1.0,2.0,3.0};
	cout<<"Adaptive simplify, dry-run:"<<endl;
	for(size_t i=0;i<5;++i)
	{
		double tolerance(aTolerance[i]);
		size_t num=cloudPrep.adaptiveSimplify(tolerance,sms,chs,true); // bDryRun=true
		cout<<"* DryRun, for tolerance="<<tolerance<<" the point cloud size would be "<<num<<endl;
	}


	cloudPrep.adaptiveSimplify(maxDiffZ,sms,chs,false);// bDryRun=false
	cout<<"CloudPrepare adaptiveSimplify, numPoints reduced="<<cloudPrep.getNumPoints()<<endl;
	Fade_2D dt;
	dt.insert(&cloudPrep,true);

	// * 4 *   Show
	write("a1","terrain_adaptiveSimplify",dt);
}


// 2: Simplify: Use a grid to cluster points
void terrain_gridSimplify()
{
	// * 1 *   Get input points
	std::vector<Point2> vInputPoints;
	getInputPoints("MOUNTAIN",vInputPoints);

	// * 2 *   Add them to a CloudPrepare object (or add them one by one)
	CloudPrepare cloudPrep;
	cloudPrep.add(vInputPoints);

	// * 3 *   Reduce the cloud size
	SumStrategy sms(SMS_AVERAGE); // or SMS_MEDIAN, SMS_MINIMUM, SMS_MAXIMUM
	ConvexHullStrategy chs(CHS_MAXHULL); // or CHS_NONE, CHS_MINHULL
	// We can use dry-run=true to determine the number of points that would result:
	double aGridLength []={1.5,2.0,2.5,3.0,3.5};
	cout<<"Grid simplify, dry-run:"<<endl;
	for(size_t i=0;i<5;++i)
	{
		double gridLength(aGridLength[i]);
		size_t num=cloudPrep.uniformSimplifyGrid(gridLength,sms,chs,true); // bDryRun=true
		cout<<"* DryRun, for gridLength="<<gridLength<<" the point cloud size would be "<<num<<endl;
	}

	double gridLength(2);
	cloudPrep.uniformSimplifyGrid(gridLength,sms,chs,false); // bDryRun=false
	cout<<"CloudPrepare gridSimplify, numPoints reduced="<<cloudPrep.getNumPoints()<<endl;
	Fade_2D dt;
	dt.insert(&cloudPrep,true);

	// * 4 *   Show
	write("a2","terrain_gridSimplify",dt);
}

// 3: Simplify: Use approxNumPoints to cluster points (also grid-based)
void terrain_numPointsSimplify()
{
	// * 1 *   Get input points
	std::vector<Point2> vInputPoints;
	getInputPoints("MOUNTAIN",vInputPoints);

	// * 2 *   Add them to a CloudPrepare object (or add them one by one)
	CloudPrepare cloudPrep;
	cloudPrep.add(vInputPoints);

	// * 3 *   Reduce the cloud size
	SumStrategy sms(SMS_AVERAGE); // or SMS_MEDIAN, SMS_MINIMUM, SMS_MAXIMUM
	ConvexHullStrategy chs(CHS_MAXHULL); // or CHS_NONE, CHS_MINHULL
	int approxNumPoints(int(vInputPoints.size()/4));
	cloudPrep.uniformSimplifyNum(approxNumPoints,sms,chs);
	cout<<"CloudPrepare numPointsSimplify, numPoints reduced="<<cloudPrep.getNumPoints()<<endl;
	Fade_2D dt;
	dt.insert(&cloudPrep,true);

	// * 4 *   Show
	write("a3","terrain_numPointsSimplify",dt);

}

// 4: Export data of the triangulation
void exportData()
{
	// * 1 *   Create a small number of input points
	std::vector<Point2> vInputPoints;
	generateRandomSurfacePoints(
			5, // numPointsX
			4, // numPointsY
			1, // numCenters (waviness)
			0,0,0,100,100,50, // Bounds xmin,ymin,zmin,xmax,ymax,zmax
			vInputPoints, // Output vector
			1  // Seed
			);

	// * 2 *   Triangulate
	CloudPrepare cloudPrep;
	cloudPrep.add(vInputPoints);
	Fade_2D dt;
	dt.insert(&cloudPrep,true);

	// * 3 *   Export to a struct
	FadeExport fadeExport; // Simple struct, see "FadeExport.h"
    bool bCustomIndices(true); // To retrieve custom indices also
    bool bClear(true); // Clear memory in dt to save memory
    dt.exportTriangulation(fadeExport,bCustomIndices,bClear);

	// All triangulation data is in 'fadeExport' now and dt has
	// been cleared during the export procedure to avoid memory
	// usage peaks: dt is empty now.

	// * 4 *   Show that all data is in fadeExport
    fadeExport.print();
    fadeExport.writeObj("a4_exportTest25D.obj");

	// NOTE: You have full access to the data in the FadeExport struct
	// and you have also access to the source code because it has been
	// implemented only in the header file "FadeExport.h". Adapt it
	// to your needs. Have also a look at the similar example in the
	// examples_2D folder.
}

// Write demo output (*.list and *.obj)
void write(const std::string& id,const std::string& name,const Fade_2D& dt)
{
	std::string outName(id+"_"+name);
	dt.showGeomview((outName+".list").c_str()); // Geomview (Linux)
	cout<<gRed("Writing "+outName+".obj")<<endl;
	dt.writeObj((outName+".obj").c_str()); // Any *.obj-Viewer
}

// Read points from a file or generate test data
void getInputPoints(const std::string& filename,
					std::vector<Point2>& vPointsOut)
{
	if(filename=="MOUNTAIN")
	{
		getMountain(vPointsOut);
	}
	else if(!filename.empty())
	{
		// Read terrain points from an ASCII file
		readXYZ(filename.c_str(),vPointsOut);
	}
	else
	{
		// No file given, create a random surface for the demo
		generateRandomSurfacePoints(
			100, // numPointsX
			100, // numPointsY
			200, // numCenters (waviness)
			0,0,0,100,100,50, // Bounds xmin,ymin,zmin,xmax,ymax,zmax
			vPointsOut, // Output vector
			1  // Seed (0 is random, >0 is static)
			);
	}
	std::cout<<"Original number of points: "<<vPointsOut.size()<<endl;
}

void getMountain(std::vector<Point2>& vPointsOut)
{
	// arr[] stores 36 kB mountain data to avoid the hassle of file paths in test environments
	float arr[]={12.737,12.8932,13.0732,13.2532,13.3856,13.5417,13.6979,13.9493,14.2246,14.4522,14.6322,14.836,15.0874,15.3626,15.6378,15.8655,16.1169,16.3683,16.5959,16.8235,17.0511,17.3025,17.5302,17.7816,18.033,18.2844,18.4644,18.6206,18.8006,19.0758,19.4701,20.0786,20.6633,21.1528,21.4042,21.489,21.5023,21.5395,21.7909,22.1138,22.3176,22.4737,22.7014,22.9766,23.2756,23.6937,24.2308,24.7917,25.4478,26.1516,26.8316,27.4401,28.0248,28.8238,30.0037,31.0408,32.0302,32.8292,33.771,34.8557,35.7023,36.5013,37.3003,37.9565,38.6602,39.3164,39.8773,40.1525,40.0944,39.7983,39.3116,38.8727,38.7193,38.7565,38.6984,38.8308,39.3441,39.8574,40.2278,40.6459,41.064,41.5297,42.1382,42.842,43.641,44.2971,44.8104,45.2285, \
	12.8492,12.9816,13.1616,13.3177,13.4501,13.5587,13.7387,13.9901,14.2653,14.4691,14.6015,14.8529,15.152,15.4272,15.6786,15.93,16.1576,16.4091,16.6605,16.8881,17.1157,17.3671,17.6185,17.8461,18.1452,18.3966,18.6242,18.8042,18.9842,19.2118,19.5823,20.0956,20.6803,21.1936,21.5402,21.6964,21.8526,21.9612,22.284,22.6069,22.8821,23.1097,23.2421,23.3506,23.5544,23.8535,24.2715,24.7134,25.2505,25.8353,26.4676,27.0761,27.6847,28.2218,29.4017,30.8197,32.1424,33.3699,34.5498,35.825,36.7192,37.661,38.5076,39.3066,40.0104,40.6665,41.2274,41.4551,41.397,41.1008,40.519,39.9847,39.6886,39.6781,39.7629,39.9429,40.361,40.8743,41.2923,41.758,42.2237,42.6894,43.2979,43.9541,44.6102,45.2187,45.5892,45.912, \
	12.9376,13.0461,13.2023,13.3823,13.5147,13.6709,13.8509,14.1023,14.3537,14.4861,14.5947,14.9175,15.2403,15.5156,15.767,15.9946,16.2222,16.4736,16.7012,16.9288,17.1803,17.4317,17.6831,17.9583,18.2335,18.5088,18.784,18.964,19.144,19.3478,19.6468,20.0411,20.5306,21.0915,21.6286,21.9752,22.1791,22.24,22.6343,23.1,23.4466,23.698,23.8542,23.9152,24.0237,24.1799,24.479,24.897,25.3627,25.8284,26.3179,26.855,27.4159,28.0006,28.8949,30.5509,32.1117,33.6726,35.281,36.699,37.7837,38.8207,39.715,40.5616,41.3129,42.0167,42.5776,42.8052,42.6995,42.3558,41.5835,40.954,40.7055,40.695,40.8274,41.1502,41.5683,42.034,42.4997,42.9653,43.431,44.0396,44.6481,45.1614,45.6747,46.0451,46.3204,46.5956, \
	12.9545,13.1107,13.2431,13.4231,13.6269,13.8069,14.0107,14.2145,14.4183,14.5983,14.8259,15.1011,15.3049,15.5563,15.8077,16.0115,16.2392,16.4668,16.7182,16.9458,17.1972,17.4724,17.7477,18.0229,18.2981,18.5733,18.8486,19.0524,19.2324,19.46,19.7114,20.0342,20.4285,20.9418,21.5265,21.9684,22.2198,22.4712,22.8893,23.4978,23.9159,24.2387,24.4425,24.5511,24.5883,24.673,24.8292,25.1759,25.6177,26.012,26.4301,26.872,27.4091,27.9938,28.9118,30.4726,32.0335,33.8324,35.6789,37.3826,38.8005,39.8376,40.8271,41.626,42.4726,43.224,43.7849,44.1078,44.1449,43.4679,42.4575,41.9233,41.8652,41.8547,41.8442,42.2623,42.9185,43.5746,43.9927,44.4107,44.8764,45.3897,45.8554,46.3211,46.7868,47.2049,47.3372,47.0887, \
	13.0191,13.1753,13.3314,13.5114,13.7391,13.9429,14.1467,14.3267,14.5066,14.7105,14.9619,15.2371,15.4409,15.6447,15.8723,16.0999,16.3275,16.5313,16.7828,17.0342,17.2856,17.5608,17.8122,18.0636,18.3627,18.6379,18.9131,19.1169,19.3207,19.5722,19.8236,20.1226,20.4216,20.8397,21.353,21.7949,22.1653,22.5596,23.0729,23.7529,24.2423,24.6842,25.0071,25.2347,25.2718,25.309,25.3461,25.5737,25.9204,26.3385,26.7565,27.1508,27.6165,28.1298,29.2145,30.4896,32.0504,33.6589,35.553,37.59,39.3413,40.6164,41.7487,42.8334,43.7752,44.5266,45.0875,45.3627,45.257,44.5323,43.5696,43.083,43.1677,43.1573,42.9563,43.3744,44.221,44.8295,45.2952,45.7609,46.2266,46.6923,47.0627,47.4808,47.9465,48.0789,47.8779,47.2009, \
	13.1313,13.3112,13.4912,13.695,13.875,14.0788,14.2588,14.4388,14.6188,14.8226,15.0741,15.3017,15.5055,15.7331,15.9607,16.1883,16.4397,16.6435,16.8949,17.1464,17.4216,17.673,17.9482,18.1758,18.4272,18.7025,18.9539,19.2053,19.4567,19.7081,19.9834,20.2586,20.5338,20.8329,21.2033,21.5738,22.0156,22.5289,23.1137,23.7698,24.3783,24.9631,25.4526,25.7992,25.9792,26.0163,26.0297,26.1383,26.3897,26.7601,27.202,27.6201,28.0382,28.48,29.5647,30.9351,32.2578,33.6282,35.5223,37.4641,38.8344,40.7286,42.5275,44.0883,45.0778,45.8291,46.4853,46.7605,46.4167,45.5968,44.6817,44.2903,44.4227,44.4598,44.4494,44.6293,45.3331,45.9893,46.5502,47.1111,47.5292,47.9472,48.3653,48.831,49.1062,48.6672,47.8473,46.7893, \
	13.2196,13.4234,13.651,13.8548,14.0586,14.191,14.3472,14.5272,14.731,14.9348,15.1624,15.39,15.5938,15.7976,16.0253,16.2767,16.5281,16.7319,17.0071,17.2823,17.5338,17.809,18.0842,18.3356,18.5394,18.7909,19.0185,19.2937,19.5689,19.8679,20.1432,20.3946,20.6698,20.8974,21.1726,21.4717,21.8659,22.4269,23.0116,23.6439,24.3715,25.0514,25.6362,26.078,26.3771,26.6047,26.7609,26.7504,26.8828,27.2532,27.7427,28.2798,29.1264,29.7826,30.2483,31.1901,32.5605,34.1213,35.9678,37.8143,39.2799,40.2694,42.2111,44.2957,46.047,47.2269,47.9307,48.0154,47.4336,46.5185,45.7938,45.5928,45.63,45.6671,45.6567,45.8843,46.5404,47.3394,47.948,48.4613,48.9269,49.2498,49.5726,49.8478,49.6945,48.7794,47.5786,46.3301, \
	13.3318,13.5832,13.8585,14.1575,14.3851,14.4699,14.5546,14.687,14.867,15.0708,15.2984,15.5022,15.6822,15.886,16.0898,16.3412,16.5927,16.8203,17.0717,17.3231,17.5983,17.8736,18.1726,18.424,18.6754,18.9506,19.1783,19.4059,19.6811,19.9801,20.2792,20.5306,20.782,21.0096,21.261,21.5601,21.9067,22.3486,22.8857,23.518,24.2218,24.9255,25.5103,26.0474,26.5607,27.0026,27.373,27.6006,27.7568,27.9368,28.4025,29.2491,30.1909,31.228,31.6461,31.9689,33.006,34.4239,36.3657,38.4027,40.2492,41.4767,42.7519,44.2651,46.1116,47.8629,48.8999,49.1752,48.4505,47.3449,46.9059,46.943,47.0754,47.0173,46.9116,47.1392,47.7954,48.5944,49.2981,49.8114,50.2295,50.4571,50.4943,50.3409,49.7115,48.6059,47.2622,46.0137, \
	13.5154,13.8145,14.1849,14.5554,14.8068,14.8915,14.8811,14.8944,15.003,15.183,15.4106,15.6144,15.8182,16.022,16.2258,16.4534,16.681,16.8848,17.0886,17.3162,17.5915,17.9143,18.2133,18.4886,18.7876,19.0866,19.3381,19.5657,19.7933,20.0685,20.3437,20.619,20.8704,21.1218,21.4208,21.6722,21.9475,22.2941,22.7598,23.3207,23.9054,24.5616,25.2415,25.8977,26.53,27.1623,27.7471,28.308,28.6784,29.1441,29.5622,30.266,31.3506,32.5305,33.52,34.2238,34.3561,34.9647,36.5255,38.5149,40.4566,42.1127,43.3878,44.9486,46.4618,48.1179,49.5359,50.1444,49.5626,48.5522,48.1608,48.3408,48.616,48.5579,48.357,48.4894,49.0027,49.7065,50.4102,51.114,51.3416,51.4264,51.1302,50.4531,49.4904,48.2895,46.9935,45.745, \
	13.7943,14.1409,14.5828,15.0485,15.2761,15.2894,15.2075,15.1256,15.139,15.2476,15.4752,15.7028,15.9542,16.2056,16.4332,16.637,16.817,17.0208,17.1532,17.3808,17.656,17.9551,18.2541,18.5769,18.8998,19.1988,19.4979,19.7493,20.0007,20.2283,20.4321,20.6597,20.9587,21.234,21.5568,21.832,22.1073,22.4063,22.7529,23.1948,23.6605,24.2452,24.9728,25.7242,26.4517,27.2269,28.0497,29.0392,29.8858,30.5895,31.1504,31.759,32.6056,33.8807,35.3463,36.5262,36.4681,36.3624,37.0186,38.5318,40.2355,42.1772,43.6904,45.6798,47.1454,48.7538,50.1718,50.8756,50.5794,49.9024,49.511,49.7386,50.0138,50.0986,49.8977,49.8396,50.21,50.7709,51.4271,52.0356,52.0728,51.8242,51.1948,50.4225,49.3169,48.0208,46.8676,45.6667, \
	14.1445,14.5626,15.0283,15.494,15.8406,15.8539,15.6768,15.4045,15.2512,15.2645,15.5159,15.8388,16.0902,16.3654,16.6168,16.8206,17.0482,17.2282,17.3844,17.5644,17.8158,18.0911,18.3663,18.6891,19.012,19.3348,19.6577,19.9567,20.2081,20.4595,20.6633,20.8671,21.1185,21.4652,21.8118,22.1347,22.4099,22.6851,22.9365,23.1642,23.487,24.0479,24.7517,25.5269,26.3735,27.2201,28.2571,29.5799,30.855,31.8445,32.7387,33.5377,34.2414,35.3737,36.9346,38.2573,38.6754,38.3792,38.6068,39.3582,40.5857,42.3846,43.8979,45.9348,48.067,49.7707,51.1411,52.1305,51.6439,50.9668,50.8135,50.9935,51.3164,51.4487,51.3431,51.0469,51.1317,51.4069,51.7773,52.0526,51.9469,51.4603,50.7832,50.1537,49.0957,47.8473,46.8369,45.8741, \
	14.4948,14.9843,15.4976,16.0585,16.4289,16.3709,16.0509,15.6119,15.2681,15.2577,15.5805,15.9748,16.25,16.5252,16.7528,17.0042,17.2318,17.4357,17.6395,17.8195,18.0471,18.2985,18.5975,18.8965,19.2194,19.566,19.9365,20.2117,20.4631,20.7145,20.9422,21.1936,21.445,21.8154,22.1859,22.5326,22.8554,23.1068,23.3106,23.443,23.5754,23.9458,24.5544,25.3057,26.1761,27.0942,28.0836,29.5968,31.0624,32.29,33.7079,34.9355,35.9249,36.9144,38.1895,39.6551,40.3112,40.8245,41.1474,41.375,41.9359,43.1634,44.5814,46.5708,48.4649,50.5495,52.6341,53.7902,53.4226,52.3646,52.1161,52.4389,52.9522,52.9418,52.7408,52.2542,51.7676,51.5667,51.461,51.4505,51.3448,51.0963,50.6097,49.9802,49.0651,47.8642,46.9015,46.0339, \
	14.9164,15.4297,15.9669,16.4563,16.8268,16.7687,16.3773,15.8907,15.4517,15.2746,15.5736,15.9917,16.3384,16.6136,16.8888,17.1402,17.3916,17.6193,17.8469,18.0983,18.3259,18.5773,18.8763,19.1754,19.522,19.9401,20.3106,20.6096,20.7896,20.9696,21.221,21.52,21.8429,22.2609,22.679,23.0495,23.3961,23.6951,23.9466,24.1266,24.1161,24.0818,24.4285,25.0132,25.8122,26.7302,27.6244,28.9472,30.3652,32.0212,33.4868,34.9048,36.2752,37.6932,39.1588,40.5291,41.5662,42.7461,43.2594,43.3442,43.7146,44.9898,46.6458,47.8257,49.3389,51.4711,53.2224,53.9309,53.7062,53.3339,53.2758,53.6463,54.1691,54.0681,53.8053,53.1283,52.3083,51.5836,51.1923,50.9913,50.7904,50.5419,50.1505,49.7115,49.0344,47.9764,47.0613,46.4318, \
	15.3619,15.899,16.3885,16.8542,17.1771,17.119,16.68,16.1219,15.6353,15.3154,15.5668,16.0087,16.3791,16.702,16.9772,17.2524,17.5038,17.7791,18.0781,18.3295,18.5809,18.8561,19.1552,19.5256,19.9199,20.3856,20.8037,21.1265,21.2589,21.3198,21.476,21.7751,22.1931,22.6826,23.1721,23.6378,24.0559,24.4025,24.7015,25.0244,25.1091,24.5749,24.4454,24.7445,25.4006,26.2472,27.07,27.8214,29.1917,30.8002,32.4086,34.0171,35.6731,37.6149,39.6519,40.927,42.2021,43.382,44.4191,45.2657,46.0647,46.8637,47.9007,49.2235,50.4986,52.4404,53.6774,53.8241,53.6613,53.6841,53.6737,53.8537,54.1241,54.3327,54.2937,53.6213,52.5634,51.553,50.8759,50.4845,50.2836,50.1303,49.8818,49.3951,48.9561,48.3267,47.602,47.2106, \
	15.8788,16.416,16.8578,17.2521,17.4559,17.3264,16.8636,16.3056,15.7713,15.3799,15.679,16.097,16.4675,16.7903,17.0894,17.3646,17.6398,17.9389,18.2617,18.5607,18.836,19.135,19.4578,19.8521,20.3416,20.8311,21.2729,21.6672,21.8234,21.7891,21.8263,22.0539,22.4958,23.0091,23.7128,24.2499,24.7394,25.1575,25.5041,25.827,25.8403,25.4728,25.1766,25.1423,25.2747,25.788,26.6108,27.4574,28.4945,30.1029,31.8066,33.7007,35.6425,37.4414,39.0498,40.563,41.9334,43.3038,44.5789,45.7588,46.8911,47.9282,49.0128,50.0975,51.3251,52.7906,53.6325,53.6554,53.6163,53.7011,53.7859,53.8706,54.0792,54.2878,54.1868,53.6383,52.6279,51.5699,50.6548,50.1206,49.9196,49.9092,49.8035,49.3645,49.0683,48.8198,48.3808,48.2751, \
	16.3243,16.8614,17.3033,17.6738,17.7585,17.51,16.9758,16.4654,16.0502,15.7778,15.9102,16.2092,16.5559,16.9025,17.2016,17.4768,17.7758,18.0987,18.4453,18.7443,19.0434,19.3662,19.7367,20.1547,20.6442,21.1575,21.6708,22.1365,22.3641,22.306,22.2242,22.3565,22.7508,23.2641,23.8964,24.5764,25.2325,25.8411,26.3782,26.7724,26.8334,26.2039,25.9554,25.8021,25.7202,25.8764,26.4135,27.1649,27.9639,29.5009,31.395,33.2891,35.1357,36.7917,38.4002,40.0086,41.5218,42.9874,44.4054,45.7758,47.0985,48.3737,49.744,50.4954,51.4372,52.5695,53.2257,53.3581,53.4428,53.6228,53.7076,53.8876,54.0961,54.2428,54.2038,53.7029,52.6449,51.5393,50.5765,49.9471,49.7937,49.688,49.63,49.5243,49.4186,49.3605,49.2548,49.4348, \
	16.6508,17.2117,17.725,18.1193,18.1326,17.765,17.2308,16.6966,16.2576,15.9852,16.0224,16.2738,16.6442,16.9671,17.2661,17.589,17.9356,18.2584,18.5813,18.9041,19.2032,19.526,19.8965,20.3621,20.9231,21.5078,22.0687,22.582,22.8334,22.8468,22.7411,22.7306,23.0058,23.4715,24.1038,24.8552,25.6304,26.3818,27.0379,27.7655,27.4455,27.0065,26.7818,26.5809,26.4038,26.3695,26.5495,26.9675,27.6713,28.8036,30.6501,32.3538,34.2479,36.2849,37.9886,39.6922,41.2531,42.5758,43.8986,45.3166,46.7822,48.2001,49.6657,50.7028,51.4066,52.2532,52.7189,52.9941,53.1741,53.3541,53.5341,53.8093,53.9893,54.1979,54.3445,53.8151,52.6142,51.461,50.4506,49.8688,49.7155,49.6098,49.5517,49.5412,49.6736,49.8536,50.1288,50.4517, \
	16.8582,17.4905,18.1229,18.5886,18.5543,18.0915,17.4858,16.9278,16.4174,16.0498,15.9917,16.3146,16.7088,17.0317,17.3545,17.6773,18.0002,18.323,18.6697,18.9925,19.3153,19.662,20.0563,20.5219,21.0829,21.6914,22.2761,22.8608,23.3027,23.3399,23.1627,23.057,23.2609,23.7027,24.2636,25.015,25.814,26.6606,27.8405,29.1157,28.629,27.9044,27.5368,27.3597,27.1825,27.0292,26.9712,27.0797,27.5454,28.3444,30.0005,31.5613,33.6935,36.1114,37.8627,39.4235,40.6987,41.8309,43.0109,44.5241,46.1801,47.6933,49.2542,50.9102,51.7568,52.1273,52.4025,52.6301,52.9053,53.133,53.313,53.5882,53.7206,53.9006,53.9377,53.4035,52.4407,51.2875,50.3247,49.7905,49.5896,49.5791,49.6163,49.7486,49.9762,50.2039,50.4315,50.8019, \
	17.0894,17.817,18.4969,19.034,19.095,18.5132,17.8599,17.278,16.7676,16.4001,16.2705,16.4505,16.7496,17.0724,17.4191,17.7419,18.0647,18.3876,18.7342,19.0809,19.4513,19.8218,20.2399,20.7055,21.1712,21.756,22.3883,23.0206,23.5815,23.7139,23.5368,23.4073,23.6349,24.053,24.5663,25.1272,25.8548,26.7014,27.9051,29.3231,29.7411,29.2069,28.7203,28.4718,28.2232,27.8556,27.6547,27.5966,27.7052,28.2661,29.7317,31.2926,33.6152,36.0807,37.5463,38.6786,39.7157,40.8956,42.2184,43.7792,45.5305,47.2818,49.2235,50.9272,51.6309,52.0014,52.229,52.409,52.6366,52.9118,53.1394,53.1766,53.309,53.4414,53.5261,53.3252,52.6481,51.4473,50.2464,49.7122,49.6065,49.6913,49.8237,50.0037,50.136,50.2684,50.3532,50.6284, \
	17.3683,18.072,18.752,19.1224,19.1358,18.7444,18.2101,17.6521,17.1179,16.7265,16.5494,16.6341,16.8618,17.1846,17.5074,17.8541,18.1769,18.5236,18.9178,19.2883,19.6349,20.053,20.4949,20.913,21.3072,21.8681,22.5481,23.2042,23.8604,24.0642,23.9109,23.829,24.1042,24.5699,25.0118,25.406,26.0146,26.6707,27.4935,29.2924,30.3771,30.4142,30.0229,30.0124,30.0972,29.7058,28.9335,28.5897,28.3888,28.6164,29.6535,31.3571,33.5369,35.5739,36.7538,37.3624,38.7803,40.2459,41.6639,43.32,45.0237,46.6797,48.5262,50.5156,51.267,51.4946,51.627,51.8546,52.225,52.6431,52.9183,53.0507,53.0402,53.125,53.2574,53.2469,52.9032,51.8452,50.4062,49.7292,49.5759,49.7082,49.8406,50.0206,50.2006,50.333,50.4178,50.693, \
	17.5042,18.208,18.7689,19.1394,19.1765,18.9042,18.5366,18.0262,17.4443,16.9101,16.6616,16.7463,16.9739,17.2968,17.6434,18.0139,18.3843,18.7786,19.1967,19.6386,20.009,20.3318,20.7737,21.2156,21.5623,22.0756,22.7317,23.4355,24.1392,24.4144,24.3564,24.3459,24.7164,25.2297,25.6239,25.9468,26.3648,26.8305,27.4152,28.6428,30.2036,31.4787,31.5635,31.6959,32.0187,31.8178,30.9502,30.1779,29.7389,29.2523,29.718,30.9932,32.7921,34.4481,35.5804,36.9984,38.464,39.9772,41.6809,43.4798,45.3263,47.1728,49.1146,50.7706,51.1411,51.1782,51.263,51.3954,51.6706,52.0887,52.602,52.8772,52.8667,52.9991,53.1315,53.121,52.8249,52.005,50.6613,49.9366,49.688,49.7252,49.9052,50.0852,50.2652,50.4452,50.6728,51.0432, \
	17.545,18.2488,18.8097,19.2039,19.3363,19.2306,18.9107,18.4241,17.7708,17.1413,16.7738,16.8109,17.0861,17.4328,17.8032,18.2213,18.6394,19.0574,19.5231,20.0126,20.4545,20.6821,21.0526,21.5182,21.9839,22.3544,22.9391,23.6191,24.418,24.7647,24.8018,24.8866,25.3761,25.9132,26.3075,26.6779,27.0722,27.5855,28.2893,29.1359,30.3158,32.448,33.4374,33.7603,34.0831,33.6917,32.6337,32.0043,31.2796,30.4597,30.2587,31.1053,32.6186,34.2746,35.7402,37.3487,38.8143,40.2799,41.8883,43.592,45.2004,46.8089,48.4649,50.0734,51.1104,50.9095,50.8514,51.0314,51.3066,51.6771,52.238,52.7513,52.9313,53.0161,53.0532,52.8999,52.5085,51.8791,50.9163,50.0488,49.7526,49.7898,49.9221,50.1021,50.2821,50.5097,50.9278,51.5363, \
	17.562,18.2895,18.8742,19.3161,19.5437,19.6047,19.3323,18.7981,18.1448,17.4916,16.9812,16.9469,17.2221,17.5926,17.9868,18.4049,18.8706,19.3363,19.8258,20.3391,20.8524,21.2466,21.4266,21.7495,22.358,22.9665,23.3608,23.8741,24.5778,24.9483,25.1283,25.4511,26.0121,26.5492,27.0387,27.7662,28.5176,29.3166,30.068,30.9146,31.7612,32.3697,33.9305,35.158,35.8142,35.6133,34.8886,34.1163,33.4868,32.4764,31.4661,31.5032,32.7784,34.3868,35.9476,37.5084,38.9264,40.2492,41.6196,43.0375,44.5031,46.064,47.72,49.3285,50.556,50.3551,50.4398,50.8103,51.1331,51.456,51.9217,52.6254,52.9483,53.0806,52.9749,52.6788,52.2874,51.8484,51.1237,50.1133,49.722,49.8067,49.9391,50.0715,50.2039,50.4791,50.9924,51.7438, \
	17.6503,18.3779,18.9864,19.4283,19.7273,19.9073,19.635,19.1008,18.4475,17.7942,17.3314,17.2733,17.4771,17.8238,18.2181,18.6123,19.0542,19.5675,20.0808,20.6417,21.1788,21.7159,22.0388,22.2902,22.6844,23.2454,23.8777,24.391,24.9281,25.3938,25.7166,26.0871,26.5528,27.2327,28.2222,29.164,30.1535,30.9524,31.7514,32.5504,33.0637,33.3866,34.2808,35.4131,36.1644,36.3921,36.3816,36.2283,35.8845,34.4932,33.2448,32.4725,33.0334,34.4037,35.8217,37.2397,38.6101,39.9805,41.3032,42.6736,44.1868,45.8428,47.4989,49.2026,49.8111,49.7054,49.9806,50.5892,51.1025,51.4729,51.7481,52.4519,53.0128,53.05,52.7062,52.3148,52.2567,52.151,51.4264,50.416,49.9294,49.9189,49.9561,50.0408,50.1732,50.3532,50.7713,51.6179, \
	17.8577,18.5615,19.1462,19.5881,19.8633,19.9957,19.7948,19.3082,18.6311,17.9778,17.5626,17.4569,17.6607,18.0312,18.4255,18.8197,19.2854,19.7749,20.3596,20.9444,21.5529,22.1376,22.6271,22.9499,23.249,23.6908,24.2994,24.9317,25.4688,25.9821,26.3764,26.723,27.2601,28.2972,29.4771,30.7046,31.6465,32.8264,33.7206,34.1863,34.5091,34.6891,35.0596,35.6205,36.1814,36.7899,37.2556,37.4356,37.4251,36.3195,34.8806,33.7274,33.6693,34.3731,35.6482,37.0186,38.4366,39.9022,41.3678,42.881,44.537,46.2407,47.9444,49.5052,49.3519,49.0558,49.6167,50.2728,50.929,51.4423,51.5746,51.8975,52.7441,53.1622,52.5327,51.9032,52.0356,52.2632,51.9671,50.9091,50.3749,50.1739,50.1159,50.0578,50.0949,50.3701,50.8834,51.7301, \
	18.1366,18.8403,19.4251,19.8669,20.166,20.2745,20.0736,19.5394,18.8861,18.2329,17.6748,17.4739,17.7729,18.191,18.5853,19.0033,19.469,20.0061,20.6147,21.2232,21.8793,22.5355,23.1202,23.5621,23.9087,24.3268,24.8401,25.4248,26.0333,26.5943,27.2504,27.7637,28.3722,29.4093,30.6844,32.2453,33.3299,34.5099,35.4993,36.0602,35.9545,35.8012,35.8384,36.066,36.3888,37.045,37.7011,38.0239,38.2039,37.5745,36.1356,34.9823,34.4481,34.6757,35.7128,37.1784,38.7392,40.2524,41.718,43.4217,45.1254,46.829,48.6756,49.7602,48.9403,48.5966,49.0622,49.6708,50.2317,50.745,51.0202,51.2954,51.6659,52.1792,52.0259,51.7297,51.5764,51.7088,51.6507,51.3546,51.0108,50.7146,50.4185,50.17,50.1595,50.3871,50.8528,51.6518, \
	18.463,19.143,19.7039,20.122,20.4448,20.601,20.3763,19.842,19.165,18.5117,18.0013,17.7765,17.9803,18.327,18.7213,19.1393,19.6288,20.1897,20.8221,21.4782,22.1582,22.8619,23.5419,24.1028,24.5685,25.0104,25.4522,25.9655,26.5026,27.5159,28.3625,29.0186,29.6748,30.6642,32.0346,33.5478,34.6801,35.7172,36.7066,37.1723,37.019,36.6752,36.6648,36.7019,36.8343,37.3476,38.0514,38.5647,38.7447,38.258,37.2477,36.1897,35.2269,34.9784,35.825,37.3858,38.9466,40.6503,42.0683,43.6767,45.428,47.2745,48.7878,49.0154,47.8621,47.7564,48.365,48.9735,49.4392,49.8573,49.942,49.9792,50.3972,50.7677,51.0429,51.0801,51.0696,51.202,51.5724,51.8001,51.8848,51.6363,51.1497,50.7107,50.5574,50.8326,51.1554,51.8116, \
	18.6466,19.3504,19.9113,20.3294,20.676,20.9274,20.7741,20.1923,19.4914,18.8143,18.2563,18.0078,18.1163,18.3916,18.762,19.2277,19.741,20.3257,20.9581,21.638,22.3656,23.0693,23.8207,24.4769,25.014,25.5035,26.0406,26.5539,27.3052,28.3423,29.3318,30.226,31.025,31.9668,33.3372,34.4695,35.2208,36.3531,37.0093,37.1893,37.3216,37.4064,37.5388,37.4807,37.4702,37.8407,38.4492,39.0101,39.333,39.2749,38.5978,37.2542,35.5772,34.9953,36.1752,38.0218,39.5826,41.191,42.609,44.0746,45.8259,47.3867,48.2333,47.9848,46.8792,47.1068,47.7153,48.0858,48.2658,48.3981,48.6258,48.7581,49.3667,49.7371,50.2028,50.5257,50.8009,51.1237,51.5418,52.0075,52.3779,52.4151,52.1189,51.7752,51.5742,51.5638,51.839,52.3999, \
	18.854,19.5102,20.0235,20.4654,20.8596,21.1587,21.1006,20.5425,19.8417,19.1408,18.5589,18.1914,18.1809,18.4561,18.8028,19.2447,19.8056,20.4379,21.0464,21.6788,22.4301,23.1339,23.8853,24.5176,25.0309,25.6156,26.367,27.4755,28.1793,28.9306,29.9677,31.0524,31.9942,33.0313,34.2112,35.153,35.8568,36.5605,36.9786,37.2062,37.3862,37.6138,37.7938,37.7834,37.9157,38.2862,38.8471,39.3604,39.7309,39.8632,39.2814,37.9377,36.5464,35.917,36.6683,38.2292,39.8852,41.6365,43.2926,44.901,46.5095,47.3561,47.4408,46.859,45.9438,46.2191,46.5895,46.8171,46.9019,47.1771,47.4523,47.68,48.3837,49.2303,49.8865,50.4474,50.9131,51.3311,51.7968,52.2625,52.6806,53.0034,53.1358,53.0777,52.8292,52.5806,52.7606,53.3692, \
	18.9662,19.5986,20.1119,20.5776,20.9956,21.3185,21.3318,20.869,20.1681,19.491,18.9092,18.494,18.3407,18.6159,18.8912,19.2616,19.8225,20.4549,21.0872,21.6719,22.3995,23.0556,23.5927,24.1537,24.6908,25.3231,26.1935,27.6829,28.5295,29.4714,30.4132,31.4979,32.5825,33.572,34.4186,35.3128,36.0166,36.6251,37.0432,37.3184,37.6412,37.9641,38.3345,38.3717,38.5993,38.8745,39.3402,39.8535,40.1763,40.2611,40.1554,38.907,37.5633,36.8386,37.1138,38.2937,40.045,41.7963,43.5476,45.2989,46.3836,46.8492,46.4579,45.7332,44.8656,45.0933,45.4637,45.6437,46.1094,46.7655,47.3264,47.935,48.353,49.2949,50.2843,50.9405,51.4538,51.9195,52.3851,52.7556,53.1261,53.5917,53.9146,54.0946,53.9889,53.6927,53.7775,54.5765, \
	19.0546,19.6393,20.1526,20.6421,21.1078,21.5259,21.6106,21.1954,20.5422,19.8651,19.2833,18.8681,18.7148,18.8948,19.051,19.3024,19.7681,20.4004,21.0565,21.6651,22.0831,22.4536,22.9193,23.504,24.1601,24.9353,25.7343,26.5095,27.4989,28.7265,30.0492,31.4196,32.6947,33.6366,34.5308,35.3774,36.0811,36.6897,37.1554,37.4782,37.8963,38.3143,38.5896,38.6267,38.8067,39.1772,39.6905,40.1562,40.3838,40.4685,40.6009,39.8762,38.5325,37.5222,37.4641,38.644,40.2524,41.8133,43.5645,45.173,45.8291,45.6758,45.1892,44.5598,44.0731,44.4436,45.0521,45.375,46.0787,47.211,48.2005,48.809,49.2747,50.2165,51.0631,51.7669,52.3278,52.8411,53.3544,53.6296,53.8572,54.0372,54.3601,54.6829,54.8629,54.8524,54.9372,55.4981, \
	19.143,19.7277,20.241,20.7067,21.1724,21.7095,21.8657,21.4981,20.821,20.1439,19.5859,19.1469,18.8508,18.9117,19.1393,19.3907,19.7374,20.2507,20.8116,21.3249,21.5049,22.042,22.5315,23.0448,23.7724,24.4999,25.2513,25.9551,26.6112,27.8864,29.3519,30.9604,32.4736,33.6059,34.4049,35.2515,36.0029,36.7066,37.2675,37.6856,38.1037,38.4265,38.6541,38.6913,38.8237,39.1465,39.6122,40.0779,40.3055,40.4379,40.5702,40.1789,39.1209,38.0629,37.5763,39.0895,40.6503,41.9731,43.4387,44.5233,44.7986,44.4072,43.8729,43.5768,43.8996,44.6986,45.7833,46.3918,47.0956,48.0374,49.2649,49.9211,50.3392,51.1858,51.9371,52.6409,53.2494,53.7627,54.3713,54.5513,54.6836,54.7208,54.9008,55.2379,55.756,56.274,56.6064,57.1245, \
	19.1837,19.7685,20.258,20.7236,21.2131,21.655,21.835,21.6341,21.1237,20.518,19.96,19.4734,18.9867,19.0001,19.442,19.622,19.8734,20.2676,20.6619,21.1514,21.6409,22.2732,22.7151,23.1808,23.7179,24.3264,24.935,25.6387,26.2711,27.4272,29.0356,30.6917,32.1573,33.2896,34.1838,35.078,35.9246,36.676,37.2845,37.8454,38.3111,38.5387,38.6711,38.7082,38.8406,39.0206,39.3911,39.8568,40.2272,40.4072,40.492,40.291,39.3759,38.556,38.3551,39.7254,40.9053,41.99,42.9318,43.5404,43.5775,43.2338,42.8424,42.8795,44.2023,45.7155,46.943,47.7896,48.4458,49.1019,49.9485,50.7951,51.6893,52.3455,53.0016,53.6578,54.3615,55.1177,55.6357,56.0919,56.1767,56.1995,56.1605,56.3691,56.7633,57.4671,58.047,58.7508, \
	18.9864,19.6426,20.2273,20.7168,21.1587,21.6243,21.971,22.0558,21.593,20.9635,20.334,19.7522,19.2894,19.3027,19.6732,20.0198,20.2713,20.5465,20.9407,21.5731,21.9197,22.5759,23.1368,23.5549,23.9967,24.51,25.1186,25.7747,26.3832,27.6346,29.0526,30.4229,31.8409,32.9732,34.0579,34.9997,35.8939,36.6453,37.2538,37.8624,38.4233,38.6509,38.7357,38.7728,38.81,38.8947,39.2176,39.6832,40.1489,40.4242,40.5565,40.4032,39.869,39.0015,38.5625,39.6471,40.4938,41.0071,41.6632,42.1765,42.166,42.108,42.1927,43.3726,45.1239,46.5419,47.9599,48.9493,49.6531,50.2616,50.8225,51.6215,52.4681,53.2671,54.0185,54.7175,55.545,56.4345,57.0145,57.4706,57.6173,57.6402,57.7249,57.8716,58.2659,58.9077,59.7353,60.5009, \
	18.9082,19.5405,20.1252,20.5909,21.0804,21.5937,22.2498,22.5012,22.0861,21.3614,20.6605,20.031,19.6158,19.5816,19.952,20.3225,20.6929,21.0872,21.5767,22.1614,22.389,22.9976,23.6299,24.1432,24.5851,25.0984,25.6831,26.2678,27.0906,28.5086,29.498,30.4399,31.6198,32.8949,34.0272,34.969,35.8633,36.567,37.1756,37.8317,38.3926,38.7155,38.8002,38.7898,38.8745,38.9593,39.2821,39.7954,40.2135,40.4887,40.5735,40.7535,40.6954,39.9707,39.0556,38.9023,39.606,39.9765,40.5374,41.0983,41.3259,41.7916,42.7334,44.0562,45.617,47.035,48.2625,49.3472,50.1938,50.9452,51.6966,52.5908,53.4374,54.1888,54.9592,55.8486,56.7381,57.5656,58.2075,58.6637,58.9341,58.8951,59.0418,59.2503,59.5827,60.2246,61.0521,61.9416, \
	18.7823,19.3432,19.8803,20.3936,20.9545,21.5392,22.243,22.542,22.2697,21.7116,21.0584,20.3813,19.8947,19.908,20.2785,20.6727,21.1384,21.6755,22.1888,22.6069,22.9059,23.5383,24.2182,24.8267,25.34,25.8295,26.3428,27.118,28.4884,29.8111,30.5625,31.0282,31.7796,32.9119,34.0442,34.9384,35.8802,36.6316,37.2877,37.8963,38.362,38.6848,38.6743,38.8543,38.9867,39.2143,39.5848,39.9552,40.2781,40.5057,40.5904,40.9133,41.0933,40.8924,40.4534,40.1096,40.242,40.3267,40.84,41.1629,41.4857,42.0466,43.0361,44.4064,45.9673,47.2424,48.2795,49.3642,50.4012,51.4383,52.4277,53.3696,54.2162,55.0771,56.0284,56.9798,57.9311,58.8206,59.4624,59.8567,60.251,60.3976,60.4824,60.5672,60.8995,61.4795,62.3071,63.1346, \
	18.6088,19.1459,19.683,20.2439,20.8524,21.4609,22.1647,22.5828,22.358,21.919,21.4086,20.7315,20.2211,20.3535,20.7478,21.1658,21.6791,22.1448,22.5391,22.8619,23.3038,24.0076,24.7351,25.4151,26.0236,26.5369,27.4311,28.5158,29.8386,31.1613,31.9603,32.1879,32.3203,32.8336,33.7278,34.7173,35.7543,36.4581,37.1142,37.7228,38.2837,38.7017,38.7865,38.8713,39.0513,39.4217,39.8398,40.2103,40.4379,40.5226,40.5598,41.0255,41.4911,41.6235,41.5655,41.4121,41.4017,41.3436,41.4284,41.2751,41.836,42.5397,43.434,44.6615,46.1747,47.4022,48.3917,49.3811,50.5134,51.7409,52.8732,53.9103,54.9473,55.9606,57.0358,58.049,59.1242,60.1374,60.8412,61.2974,61.7535,62.024,62.1087,62.1316,62.4021,62.982,63.8096,64.5134, \
	18.4352,18.9724,19.5571,20.1656,20.7503,21.3589,21.9912,22.3617,22.1607,21.8408,21.4018,20.9152,20.619,20.8704,21.3837,21.7542,22.1008,22.4237,22.7703,23.2836,23.7731,24.3578,25.0378,25.7891,26.5643,27.7442,28.7337,29.7708,31.0459,32.6067,33.12,33.1096,33.0515,33.3267,33.9828,34.6866,35.438,36.0941,36.7503,37.454,38.0626,38.5282,38.7082,38.7454,39.0682,39.4863,39.9996,40.3701,40.6929,40.9205,41.1005,41.6138,42.0319,42.3547,42.4871,42.5242,42.4662,42.2176,42.2548,41.9586,42.2815,42.9376,43.7842,44.9641,46.2869,47.5144,48.5515,49.4933,50.6256,51.8055,52.9378,54.0701,55.3357,56.6585,57.7955,58.8707,60.0696,61.2067,62.158,62.738,63.318,63.9598,63.797,63.6341,63.9046,64.4846,65.1883,65.7683, \
	18.3332,18.9417,19.5264,20.1111,20.6482,21.1615,21.6748,21.9263,21.8444,21.7625,21.5378,21.1464,20.9454,21.5064,21.9959,22.152,22.4035,22.8215,23.311,23.8719,24.4805,25.0176,25.5547,26.187,27.3907,28.8087,30.0362,31.3114,32.0628,33.0522,33.5655,33.6027,33.6874,34.0103,34.4759,34.894,35.3597,35.9206,36.4339,36.9472,37.4605,37.9262,38.2967,38.6195,39.0376,39.5509,40.1118,40.6251,41.1384,41.5564,41.9745,42.3926,42.8107,43.1335,43.4087,43.5411,43.4354,42.9964,43.0336,43.0231,43.0126,43.2879,44.0392,45.1715,46.4467,47.579,48.5684,49.5579,50.5473,51.632,52.7643,53.8966,55.2289,56.7373,57.9982,59.1352,60.3961,61.6569,62.794,63.7453,64.9443,65.7099,65.2995,64.6415,64.4787,64.6872,65.3291,66.3423, \
	18.4215,18.9825,19.4958,20.0091,20.4985,20.9404,21.2156,21.348,21.409,21.5652,21.6737,21.6871,21.748,21.9757,22.0842,22.3118,22.7061,23.2194,23.7803,24.4365,25.0926,25.6535,26.143,26.942,28.1695,29.5875,31.0055,32.233,32.9368,33.5929,33.9634,34.0481,34.1329,34.3605,34.6834,35.0062,35.4243,35.89,36.1652,36.3928,36.7156,37.3718,38.0755,38.7317,39.3402,39.9487,40.5097,41.0706,41.7267,42.2876,42.8009,43.219,43.6847,44.0551,44.3304,44.4627,44.2618,43.8704,43.7171,43.9447,43.9819,43.9238,44.2943,45.1885,46.4636,47.5007,48.4425,49.3844,50.3738,51.4109,52.4955,53.7231,54.9982,56.4448,57.8294,59.0903,60.2273,61.3025,62.3157,63.4528,64.7755,65.7269,65.3784,64.5965,64.248,64.2709,64.9127,65.7403, \
	18.4623,18.9756,19.4413,19.8355,20.1822,20.4812,20.6612,20.865,21.0688,21.3679,21.6669,21.8707,22.0983,22.3497,22.5059,22.7811,23.1516,23.6649,24.2496,24.9296,25.5857,26.218,27.2075,28.1493,29.1864,30.4139,31.7367,32.7737,33.3347,33.8956,34.266,34.3508,34.3879,34.4727,34.6051,34.8327,35.2508,35.9069,35.8012,36.0764,36.2564,37.3411,38.3782,39.2248,39.9762,40.6799,41.2884,41.8494,42.4103,42.9712,43.6273,44.093,44.5587,44.9768,45.252,45.3368,45.0882,44.7445,44.2578,44.1521,44.1893,44.4169,44.9302,45.634,46.3377,46.9463,48.1262,49.2585,50.2955,51.285,52.2744,53.3115,54.3962,56.1522,57.6607,59.0453,60.2443,61.3194,62.2708,63.0983,64.1735,64.7535,64.343,63.7469,63.4603,63.7307,64.4345,64.7669, \
	18.4554,18.8735,19.2202,19.4954,19.7468,19.9744,20.2496,20.6201,21.0144,21.48,21.9219,22.3162,22.639,22.8904,23.1181,23.3695,23.6923,24.158,24.7427,25.3989,26.055,27.1873,28.3672,29.4043,30.2985,31.1451,31.9917,32.5526,32.8278,33.484,34.3306,34.4154,34.3573,34.3944,34.2887,34.2783,34.4583,34.9716,35.4849,36.2839,37.0828,38.2151,39.2046,40.0036,40.7549,41.4587,42.1625,42.7234,43.1891,43.7024,44.4061,44.967,45.4327,45.8508,46.1736,46.2584,46.0575,45.5709,44.8938,44.7405,44.73,44.91,45.3757,46.0318,46.8308,47.487,47.905,48.7993,49.7887,50.8258,51.9105,53.0428,54.475,56.1692,57.8633,59.3718,60.6326,61.7078,62.4734,62.9915,63.4477,63.4705,63.0601,62.5878,62.5487,63.1287,63.7087,63.6696, \
	18.4248,18.7476,18.9752,19.1314,19.2876,19.5866,20.0285,20.5894,21.1503,21.7351,22.3436,22.8569,23.2512,23.5264,23.754,24.0054,24.3044,24.7225,25.2596,25.892,26.8814,28.1566,29.4317,30.6592,31.1725,31.4477,31.4849,31.522,31.4163,31.7868,32.9667,33.8609,33.7552,33.4115,33.2582,33.4382,33.9991,34.8457,35.8827,36.9198,37.9092,38.9939,40.031,40.9728,41.7242,42.3803,42.9413,43.5498,44.0155,44.3383,45.0421,45.7934,46.3544,46.9153,47,47.0372,46.7887,46.2068,45.625,45.2812,45.0327,45.1174,45.4879,46.1916,46.9906,47.7896,48.3505,49.0543,49.9485,50.9856,52.0703,53.3454,54.6777,56.4337,58.1279,59.6982,61.0829,62.158,62.6761,62.8847,62.8456,62.3733,61.8391,61.4286,61.5753,62.2791,62.7971,62.6343, \
	18.3227,18.5503,18.6589,18.7913,19.0189,19.4131,20.0217,20.7016,21.3578,22.0139,22.6939,23.2786,23.7681,24.1147,24.4138,24.6652,24.9404,25.3109,25.8242,26.7184,27.9459,29.1734,30.6866,31.3428,31.2847,30.9885,30.4543,30.0153,29.9096,30.2325,31.3171,32.0685,31.9628,31.6191,31.9895,33.3123,34.635,35.434,36.4711,37.5557,38.5928,39.6299,40.6669,41.7516,42.5982,43.2068,43.7677,44.281,44.699,45.0219,45.4399,46.3342,47.1808,47.9798,47.8265,47.6731,47.3294,46.7951,46.1181,45.6315,45.3353,45.2772,45.6001,46.3038,47.1028,47.8542,48.4627,49.2141,50.2035,51.2406,52.3253,53.6004,55.0041,56.7602,58.4543,60.0247,61.4093,62.2369,62.5073,62.4064,61.996,61.2142,60.7418,60.3933,60.7876,61.5532,62.0094,61.9703, \
	18.1016,18.2578,18.3425,18.5463,18.9644,19.5253,20.2291,20.909,21.589,22.2689,22.9251,23.5812,24.1421,24.6554,25.0259,25.3249,25.5763,25.8992,26.6982,27.8781,29.1056,30.5236,31.2273,30.8836,30.1113,29.4818,28.9476,28.4134,28.2601,28.5353,29.4295,29.9428,29.9799,30.3504,31.6732,33.1864,34.7948,36.0223,37.107,38.2393,39.2764,40.2658,41.3029,42.2923,43.1389,43.8427,44.4988,45.0598,45.4778,45.6102,45.933,46.5892,47.2453,47.9015,48.1291,48.1186,47.6796,47.0026,46.2303,45.8389,45.6379,45.6751,45.9979,46.5588,47.215,47.8235,48.4321,49.1834,50.2681,51.3528,52.4375,53.7126,55.3306,57.1485,58.8427,60.3511,61.4263,61.9443,61.9672,61.5568,60.8987,60.1169,59.8303,59.9151,60.4331,61.3226,61.9025,61.9254, \
	17.809,17.9414,18.0976,18.4919,19.1004,19.8042,20.5317,21.2355,21.9154,22.5716,23.2277,23.8839,24.4686,25.0057,25.4952,25.9847,26.4504,26.916,27.8103,28.9426,29.932,30.5405,30.4825,29.3292,28.6046,28.0227,27.4885,27.0495,26.8962,27.0762,27.7323,28.198,28.6637,29.7007,31.2616,32.7272,34.288,35.6108,36.8383,38.2086,39.5314,40.7589,41.7484,42.6426,43.632,44.3834,45.0396,45.6481,46.1138,46.1985,46.2833,46.6538,47.2147,47.7756,48.146,48.326,47.7918,46.8767,46.1044,45.9034,45.9406,46.1206,46.3958,46.8615,47.3272,47.8405,48.4014,49.1052,50.2374,51.465,52.5973,53.7772,55.5951,57.4131,59.1072,60.6157,61.3813,61.528,61.3652,60.9547,59.9253,59.0197,59.1664,59.8701,60.8215,61.649,62.1671,62.19, \
	17.4689,17.7441,18.1384,18.7231,19.403,20.1068,20.8582,21.5381,22.1943,22.8742,23.5304,24.1627,24.7474,25.3321,25.8693,27.0015,27.8482,28.4567,28.97,29.769,30.0442,29.9385,29.071,27.9177,27.0978,26.6112,26.3865,26.2808,26.2465,26.4027,26.7017,26.977,27.7283,29.0511,30.5643,31.9823,33.4479,34.8183,36.141,37.7018,39.3103,40.8235,41.9558,42.8976,43.8395,44.5432,45.1041,45.7127,46.226,46.4536,46.3003,46.6231,47.2316,47.7449,48.1154,48.2954,47.5231,46.5127,45.9785,45.968,46.1004,46.2804,46.5556,46.9261,47.2965,47.7146,48.1803,49.0745,50.2544,51.4819,52.7094,53.937,55.674,57.4919,59.1242,60.6326,61.0269,60.8022,60.3917,59.8575,59.0757,58.4176,58.9357,60.2585,61.5812,62.5945,63.1125,63.0735, \
	17.2001,17.7372,18.3934,19.1924,19.9199,20.5523,21.1132,21.7931,22.4731,23.2007,23.8806,24.5606,25.1929,25.8491,26.648,27.5423,28.5793,29.7116,29.844,30.1668,29.8707,28.8603,27.5642,26.5539,26.1863,25.8901,25.6892,25.6073,25.6207,25.7769,26.0997,26.5416,27.1739,28.4967,29.9146,31.285,32.703,34.0734,35.4437,37.0046,38.7082,40.3643,41.7823,42.867,43.6183,44.1792,44.5973,44.9202,45.2906,45.5182,45.9363,46.5924,47.2486,47.8571,48.1323,47.8838,47.0163,46.1963,45.9002,45.985,46.165,46.2973,46.5249,46.8002,47.123,47.4459,47.8639,48.8534,50.2237,51.5465,52.7264,53.9539,55.5052,57.1994,58.7078,59.5973,59.744,59.4573,58.9231,58.3889,57.7308,58.1251,59.2622,60.7706,62.2172,63.3542,64.2437,64.4522, \
	17.5028,18.1351,18.8151,19.495,20.1274,20.7597,21.4396,22.0958,22.7757,23.5509,24.3261,25.0061,25.5908,26.485,27.1888,27.8925,28.5963,29.4429,29.9086,29.8981,29.0782,27.8297,26.3908,25.928,25.5366,25.2167,25.0157,24.9577,25.0662,25.3177,25.7833,26.4157,27.0956,28.1803,29.4555,30.7306,32.0533,33.4713,34.8893,36.4501,38.011,39.5718,40.9898,42.3125,43.1115,43.4344,43.7572,44.1753,44.5457,45.0114,45.7152,46.5142,47.1703,47.5884,47.7208,47.3294,46.6523,46.0228,45.9171,46.0495,46.1819,46.3619,46.5419,46.7219,46.9495,47.32,47.8809,48.7275,50.0978,51.373,52.6005,53.828,55.027,56.5973,57.982,58.2525,58.0277,57.6173,57.2069,56.9202,57.005,58.204,59.5886,61.159,62.6055,63.9283,65.0653,65.5834, \
	17.9959,18.5568,19.1891,19.7739,20.3824,21.0861,21.8137,22.4222,23.0546,23.7821,24.4621,25.0468,25.5601,26.2639,26.92,27.5286,28.1371,28.8408,29.2589,28.9151,27.8095,26.5135,25.8126,25.3022,24.9108,24.5908,24.3899,24.427,24.6785,25.0965,25.7289,26.385,27.0412,27.9592,29.0439,30.1762,31.4513,33.0121,34.6682,36.1338,37.3613,38.6364,40.0068,41.3296,42.1286,42.3086,42.7266,43.3352,43.9913,44.7903,45.6845,46.3883,47.0444,47.272,47.2139,46.8702,46.3359,45.9922,45.9817,46.1141,46.2465,46.3789,46.5588,46.6912,46.9665,47.2893,47.7074,48.3159,49.6863,51.009,52.1889,53.2736,54.3583,55.6239,56.5134,56.5362,56.1877,55.9011,55.7382,56.0087,57.0839,58.5923,60.1627,61.6092,63.0558,64.3785,65.7013,66.6526, \
	18.3699,18.8832,19.4442,19.9813,20.6136,21.3412,21.9973,22.5344,23.1191,23.7039,24.1934,24.6352,25.0771,25.4714,25.7942,26.2599,26.8208,27.715,28.0379,27.3608,26.1838,25.6972,25.2105,24.7239,24.3326,23.9888,23.8593,24.0155,24.4573,24.923,25.4125,25.8782,26.4391,27.1191,28.2514,29.955,31.4206,32.8862,34.0661,35.1032,35.9974,37.225,38.8334,40.2514,40.7171,41.0875,41.7437,42.6379,43.6273,44.5692,45.3682,46.0243,46.7757,47.0509,46.7547,46.3634,46.0672,45.9615,45.9987,46.1787,46.311,46.4434,46.5758,46.8034,47.1263,47.4015,47.7243,48.2376,49.3699,50.5022,51.4917,52.4335,53.4229,54.3648,54.7971,54.7724,54.5715,54.6562,55.1981,56.0875,57.7198,59.2901,60.8605,62.3071,63.6917,65.0145,66.3991,67.66, \
	18.5774,19.0669,19.5563,20.0696,20.702,21.3581,21.8952,22.3609,22.8266,23.2447,23.6627,24.0332,24.4037,24.7503,24.9779,25.2769,25.6474,26.2083,26.4835,26.0683,25.5817,25.1189,24.7514,24.3362,23.8734,23.5296,23.4715,23.7229,24.1648,24.5114,25.0009,25.6333,26.4561,27.2789,28.6492,30.2101,31.1519,31.7604,32.5118,33.6441,34.9192,36.3848,37.9933,39.0303,39.2103,39.8665,40.9512,42.1787,43.311,44.2052,44.909,45.6603,46.3165,46.4965,46.2956,45.9994,45.9413,45.9785,46.0632,46.2432,46.3756,46.508,46.7356,47.0584,47.4289,47.6565,47.9317,48.3974,49.1488,49.9002,50.5563,51.3077,52.1543,52.9533,53.3713,53.4561,53.4933,54.3399,55.5245,57.033,58.6652,60.2975,61.744,63.1287,64.4515,65.7742,67.1589,68.1721, \
	18.6657,19.1076,19.5733,20.0628,20.5999,21.1132,21.5789,21.9731,22.3436,22.7141,23.0845,23.4074,23.6826,23.934,24.1616,24.4606,24.8787,25.2968,25.5244,25.3949,25.0035,24.4931,24.1493,23.8293,23.4618,23.1894,23.1313,23.3352,23.7294,24.2903,25.1607,25.8645,26.5921,27.3434,28.571,29.608,30.0737,30.5394,31.386,32.6611,34.3172,35.878,37.0103,37.5236,37.7512,38.8359,40.2539,41.6719,42.8518,43.7936,44.545,45.344,45.9049,45.9897,45.9792,45.9687,46.0059,46.0907,46.1754,46.3554,46.3926,46.5726,46.8954,47.3611,47.8268,48.0544,48.1868,48.462,48.7372,48.7267,49.3829,50.1343,50.9333,51.4466,51.8646,52.1875,53.2721,54.6901,56.3462,57.9784,59.6726,61.4286,62.7514,64.0741,65.4588,66.4102,66.8663,66.394, \
	18.5875,19.0531,19.4474,19.8893,20.3312,20.7254,21.1197,21.4901,21.8368,22.1834,22.5063,22.7577,22.9853,23.1891,23.3929,23.6919,24.1576,24.5757,24.8509,24.7928,24.4015,23.9387,23.5473,23.2749,23.0264,22.8493,22.8864,23.1378,23.5797,24.2359,25.0587,25.8338,26.5138,27.0985,27.6356,28.1965,28.6146,29.366,30.6411,32.1543,33.858,35.1808,35.8369,36.1121,36.673,38.091,39.6995,41.1175,42.3926,43.4773,44.3239,45.1229,45.6362,45.7685,45.9485,46.0333,46.2133,46.2505,46.3828,46.42,46.4571,46.6371,47.0552,47.6161,48.0342,48.2618,48.2037,48.0504,47.6114,47.6009,48.2571,49.2465,49.9027,50.416,51.0721,52.252,53.7652,55.288,57.044,58.8001,60.5561,62.2503,63.6349,65.2672,66.528,66.5509,66.1405,64.554, \
	18.3663,18.7368,19.0358,19.4063,19.7529,20.1234,20.5176,20.9119,21.2586,21.5814,21.809,22.0128,22.2404,22.4918,22.7194,23.0423,23.4604,23.807,24.0822,24.0718,23.847,23.4557,23.0643,22.7681,22.6148,22.5805,22.7129,23.0596,23.5491,24.1576,24.8851,25.5889,26.126,26.5441,26.8907,27.1898,27.6078,28.6211,30.2295,31.8856,33.3512,34.1502,34.5683,34.9387,35.9758,37.489,39.0022,40.5154,41.981,43.2561,44.2932,45.0446,45.4627,45.7855,45.8226,46.0026,46.1826,46.2674,46.3522,46.4369,46.5217,46.7017,46.9293,47.3474,47.7178,47.755,47.3636,46.7341,46.5808,47.3322,48.5121,49.5016,50.1101,50.7186,51.7557,53.0784,54.4488,56.0477,57.8657,59.7455,61.5635,63.01,64.3328,65.5936,66.2355,65.6394,64.2385,62.4044, \
	17.6691,18.0633,18.3862,18.709,19.0318,19.4023,19.8442,20.2146,20.5613,20.8603,21.0403,21.1727,21.4241,21.747,22.0698,22.4164,22.7869,23.1335,23.3612,23.4459,23.2926,22.9726,22.6051,22.3565,22.2508,22.3356,22.6108,23.0051,23.447,23.9841,24.5688,25.1535,25.5478,25.8468,26.2173,26.7068,27.3629,28.3524,29.9608,31.5692,32.5111,32.9768,33.3948,34.1938,35.5642,36.8393,38.3049,40.0086,41.7123,43.1303,44.2149,45.0139,45.4796,45.7072,45.9348,46.1148,46.1996,46.2844,46.4167,46.4539,46.3006,46.1949,45.994,45.9835,46.0683,46.1054,45.9045,45.7512,46.5026,47.6825,49.0528,50.1375,50.9841,51.7355,52.6297,53.8572,55.2371,56.9932,58.8111,60.6291,62.1994,63.3365,64.0402,64.744,64.8907,63.985,62.2747,60.3787, \
	17.1384,17.4851,17.7603,18.0594,18.3584,18.6812,19.0993,19.5174,19.864,20.1392,20.343,20.4992,20.7268,21.0497,21.3963,21.743,22.1372,22.4363,22.6877,22.8201,22.7382,22.4658,22.1459,21.9688,21.9583,22.1145,22.4849,22.9268,23.3211,23.6915,24.0858,24.5515,24.9695,25.3638,25.8533,26.5094,27.3322,28.4169,30.0254,31.0624,31.5281,31.9462,32.7452,34.0203,35.3907,36.8087,38.5123,40.2636,41.8721,43.2901,44.3747,45.0309,45.4489,45.7718,45.9994,46.1318,46.1213,46.0632,46.1956,46.0899,45.4128,44.6882,44.2968,44.1911,44.3711,44.5511,44.9691,45.7205,46.8528,48.2232,49.5935,50.7735,51.7153,52.5619,53.5037,54.5884,56.1207,57.8767,59.6328,61.1412,62.0926,62.6725,62.7573,63.0278,62.9887,62.2069,60.6204,58.5387, \
	16.6793,17.0021,17.2535,17.5049,17.7563,18.0316,18.402,18.8439,19.1667,19.442,19.6696,19.8972,20.1724,20.4238,20.7705,21.1409,21.4876,21.7866,22.0618,22.1942,22.1361,21.9352,21.7105,21.6286,21.7372,21.9886,22.4067,22.8247,23.1476,23.3752,23.6266,24.0685,24.5818,25.1903,25.8703,26.6454,27.4682,28.8624,29.8519,30.2223,30.5928,31.4394,32.8097,34.4182,35.979,37.4446,39.0531,40.7567,42.2699,43.4975,44.4393,44.905,45.2278,45.884,46.1592,45.9107,45.3764,45.0803,45.1174,44.9165,44.1918,43.3243,42.7901,42.9224,43.5786,44.33,45.2718,46.3565,47.5364,48.8591,50.0867,51.2189,52.3036,53.3407,54.2825,55.372,56.8804,58.5746,60.083,61.1582,61.5524,61.3896,61.103,61.1258,61.0868,60.6145,59.2756,57.07, \
	16.3153,16.5905,16.8181,17.0457,17.2495,17.501,17.8,18.1704,18.4695,18.7209,18.9723,19.2951,19.5942,19.8694,20.2161,20.6103,20.9332,21.1846,21.3646,21.497,21.4865,21.357,21.2751,21.3361,21.5875,21.9341,22.2808,22.6274,22.9264,23.2255,23.5959,24.1569,24.7892,25.4929,26.2919,27.0909,27.8423,28.7365,28.6784,29.3346,30.4193,31.742,33.3028,35.0065,36.7578,38.4139,39.9271,41.3451,42.6202,43.7049,44.2182,43.922,43.9592,44.3772,44.7477,44.3087,43.8221,43.6212,43.7535,43.6479,42.7803,41.9128,41.8547,42.6061,43.8336,45.0135,46.241,47.3733,48.5532,49.6855,50.675,51.6168,52.7491,53.929,54.8232,56.0079,57.5164,59.0867,60.2238,60.6799,60.579,59.9829,59.3867,59.4096,59.3706,58.8363,57.745,55.9728, \
	16.0465,16.2504,16.4303,16.6103,16.8141,17.0418,17.2694,17.5446,17.8912,18.0236,18.2512,18.6931,19.0636,19.3388,19.6378,20.0321,20.3787,20.654,20.8101,20.8473,20.7892,20.7549,21.0064,21.2816,21.5568,21.8797,22.1787,22.5015,22.8482,23.2662,23.8034,24.4357,25.0918,25.748,26.4517,27.1793,27.6688,27.8012,28.0764,29.3039,30.7695,32.3304,33.9864,35.7853,37.489,39.145,40.6106,41.8382,42.8276,43.3885,43.2828,42.6534,42.6429,42.9181,42.8601,42.5639,42.2201,42.1144,42.2944,42.284,41.3688,40.7394,41.3003,42.7183,44.3743,45.7923,47.1151,48.3426,49.5701,50.5596,51.4538,52.2052,53.0518,54.1364,55.2592,56.3963,57.9666,59.2275,59.8693,59.7065,59.0485,58.3904,57.4848,57.322,57.1591,56.8106,55.9669,54.9898, \
	15.7778,15.9816,16.0902,16.2702,16.5454,16.8206,16.9768,17.252,17.6463,17.8025,17.9349,18.2815,18.652,18.9272,19.2024,19.5729,19.8957,20.1471,20.3271,20.4357,20.4967,20.6529,20.9995,21.4176,21.8118,22.1109,22.3861,22.6375,22.8889,23.2356,23.7727,24.2622,24.8469,25.503,26.2068,26.8153,27.1858,27.4134,27.9981,29.559,31.2626,33.0139,34.8128,36.5165,38.1249,39.6858,40.9609,42.0932,42.8446,42.6436,42.0142,41.4323,41.5647,41.8399,41.7343,41.3429,40.9515,40.6553,40.7401,40.682,39.9097,39.8993,40.9839,42.64,44.4389,46.0473,47.4177,48.8357,50.1108,51.1003,51.9945,52.7935,53.4496,54.201,55.4619,56.537,57.7979,58.4397,58.4626,57.7427,56.8989,56.4266,55.9543,55.1915,55.181,55.123,54.7792,54.1021, \
	15.6281,15.9986,16.1786,16.43,16.7766,17.0757,17.3271,17.6737,17.9966,18.0575,18.0709,18.2509,18.5023,18.7061,18.9575,19.2089,19.508,19.7356,19.9156,20.1432,20.4184,20.765,21.2783,21.7678,22.2097,22.5802,22.8554,23.083,23.2868,23.562,23.9801,24.3506,24.84,25.3295,25.8666,26.3799,26.8218,27.3589,28.2532,29.9568,31.6605,33.507,35.3535,37.0096,38.4752,39.9408,41.0731,42.1101,42.3378,41.6131,40.5551,40.0208,40.2485,40.5713,40.4656,40.2647,39.8257,39.3391,38.9477,38.7468,38.2601,39.2496,40.858,42.4189,43.9797,45.4453,46.8633,48.3289,49.6516,50.7363,51.6781,52.6676,53.5618,54.3132,55.1693,56.7397,58.1862,57.9615,57.0559,55.9645,55.1446,54.8008,54.5999,53.7324,53.7695,53.7591,53.4629,52.9287, \
	15.8593,16.1822,16.4574,16.7326,17.0555,17.3783,17.725,18.0954,18.3706,18.4316,18.4211,18.4583,18.5907,18.7468,18.9268,19.1068,19.3583,19.5859,19.8135,20.1601,20.602,21.1153,21.6762,22.1895,22.7028,23.1685,23.5151,23.7666,23.9704,24.2218,24.5446,24.8199,25.2617,25.5846,25.8836,26.3017,26.8388,27.5902,28.7463,30.3547,31.9155,33.6192,35.3229,36.8837,38.3969,39.9101,41.09,41.7462,41.0691,39.9159,38.8579,38.2284,38.4084,38.8265,39.0065,38.9008,38.4142,37.9276,37.6314,37.3353,37.6581,38.9332,40.3512,41.7216,43.0443,44.3671,45.9279,47.4411,48.7639,49.9438,50.9809,52.1132,53.1502,54.1873,55.3101,56.509,57.2747,56.4929,55.4015,54.4578,53.7807,53.4846,53.1884,52.5114,52.4057,52.3476,52.0514,51.7077, \
	16.0667,16.4134,16.7362,17.0353,17.3581,17.7048,18.0514,18.3742,18.6495,18.7342,18.7238,18.7133,18.7743,18.8828,19.039,19.1952,19.399,19.6504,19.9971,20.439,20.9523,21.537,22.1217,22.6826,23.2435,23.7568,24.1987,24.5215,24.7968,25.0482,25.3948,25.5272,25.9929,26.2919,26.4719,26.771,27.2843,27.8928,29.1917,30.705,32.2182,33.6838,35.1494,36.5673,37.9377,39.5938,40.8689,40.8108,39.7528,38.552,37.6369,36.9598,36.9969,37.2722,37.4522,37.2512,36.9075,36.4208,36.5532,36.9713,37.9131,39.3311,40.511,41.6909,42.8708,44.146,45.6592,47.0296,48.3999,49.7227,50.9026,52.0825,53.2624,54.3947,55.8222,56.6498,56.425,55.2099,54.1995,53.2844,52.6073,52.1683,51.8246,51.4332,51.0894,50.8885,50.6875,50.3914, \
	16.2504,16.6208,16.9675,17.3141,17.6369,17.936,18.2588,18.5578,18.8331,18.9893,19.0026,18.9683,18.9817,19.0664,19.2226,19.4026,19.6064,19.9055,20.3473,20.8368,21.3501,21.9349,22.5672,23.1995,23.8081,24.3452,24.8108,25.1813,25.5756,26.0413,26.5307,26.4489,26.7955,27.2374,27.3698,27.4545,27.6583,28.2192,29.3515,30.8171,32.1875,33.4626,34.7854,36.1558,37.5737,39.0393,39.6955,39.0184,38.2461,37.4262,36.4634,35.5483,35.4426,35.7178,35.9454,35.6017,35.3531,35.1522,36.2369,37.512,38.8824,40.3004,41.4327,42.5173,43.602,44.7819,46.1523,47.5227,48.8454,50.1682,51.3957,52.6708,53.8984,55.0116,56.2106,56.6667,55.6992,54.4174,53.2642,52.2538,51.4815,50.8997,50.4607,50.1645,49.4398,49.2389,49.1808,48.9799, \
	16.3387,16.7806,17.1749,17.4977,17.7967,18.072,18.3472,18.67,18.9929,19.2443,19.3052,19.2948,19.2605,19.2977,19.4538,19.7053,19.9805,20.3509,20.7928,21.2823,21.8194,22.428,23.0841,23.6926,24.2774,24.8145,25.304,25.7458,26.1639,26.6296,27.0715,27.3943,27.5743,27.7543,27.7676,27.8286,27.9372,28.379,29.2256,30.5008,31.7283,33.0511,34.3738,35.7442,37.1146,38.0564,38.0935,37.3212,36.7394,36.1099,35.2424,34.3749,33.9835,34.1159,34.4387,34.4759,34.7987,35.5977,36.9681,38.386,39.8993,41.3648,42.7352,43.8675,44.857,45.894,47.1215,48.4919,49.7671,51.0898,52.3649,53.6401,54.82,56.019,56.537,56.3123,54.921,53.6725,52.4717,51.4137,50.4509,49.7262,49.192,48.6578,47.6474,47.3989,47.5313,47.5684, \
	16.3795,16.869,17.2632,17.5623,17.8137,18.0413,18.2927,18.6156,19.0336,19.4993,19.7983,19.7879,19.6346,19.5765,19.7803,20.0793,20.4736,20.9631,21.405,21.8468,22.3363,22.8972,23.482,24.0667,24.6514,25.2123,25.8447,26.2627,26.6332,26.8608,27.0884,27.3874,27.6151,27.7712,27.856,27.8932,27.9779,28.2055,28.9093,29.994,31.2215,32.4966,33.7718,34.9993,36.0364,36.502,36.1107,35.4812,35.1374,34.7461,34.0214,33.2015,32.7625,32.7996,33.1701,33.8262,34.768,36.1384,37.6992,39.2601,40.8209,42.5246,44.0854,45.2653,46.3024,47.2442,48.3765,49.6516,50.8792,52.2019,53.4294,54.6093,55.7654,56.4692,56.6159,55.5245,54.1285,52.9753,51.822,50.764,49.7061,48.7433,48.0186,47.7224,46.7597,46.2255,46.2626,46.4426, \
	16.3726,16.8859,17.2802,17.5792,17.7592,17.9392,18.1668,18.5849,19.1696,19.802,19.982,19.9953,19.9848,20.022,20.2258,20.5724,21.0619,21.5752,22.0409,22.5066,22.9247,23.3427,23.8084,24.3693,25.0017,25.753,26.5044,26.5654,26.7216,26.7825,26.9387,27.2139,27.5368,27.693,27.7777,27.8149,27.8997,28.0796,28.6882,29.5824,30.7623,31.9422,33.0269,34.0163,34.6725,34.7573,34.1754,33.7364,33.5831,33.3346,32.7527,32.028,31.7319,32.0547,32.8061,33.8908,35.2135,36.6791,38.2876,39.9912,41.6473,43.5414,45.1023,46.4726,47.6049,48.4515,49.3457,50.5733,51.8008,53.0759,54.2082,55.1405,55.5967,55.0006,54.5473,54.1559,53.4312,52.3732,51.2676,50.2096,49.1516,48.046,46.9404,46.3586,46.0624,45.3377,44.8035,44.793, \
	16.2944,16.8077,17.2495,17.5248,17.6809,17.8133,18.0886,18.578,19.1866,19.6761,19.9275,20.0837,20.2399,20.396,20.6475,21.0179,21.5312,22.0683,22.6054,23.0711,23.4654,23.8596,24.2777,24.791,25.3757,26.0795,26.5928,26.6061,26.5957,26.6328,26.8366,27.088,27.3633,27.5195,27.628,27.6652,27.75,27.9538,28.4194,29.3137,30.3507,31.3402,32.0915,32.7001,33.1181,33.0125,32.5258,32.2297,32.124,31.9707,31.5317,30.9975,30.987,32.0241,33.442,34.6696,35.9447,37.3151,38.7807,40.4843,42.3309,44.1774,45.881,47.299,48.098,48.6113,49.3151,50.495,51.7225,52.9977,53.9395,54.0243,53.7281,52.9558,52.8025,52.5539,52.353,51.5331,50.618,49.6552,48.5972,47.3964,46.2431,45.3756,44.889,44.2119,43.5825,43.6196, \
	16.1923,16.658,17.0522,17.3513,17.4836,17.616,17.9389,18.4998,19.0369,19.5026,19.8968,20.172,20.4711,20.7225,21.0453,21.4634,21.9529,22.49,23.0271,23.4928,23.9347,24.3527,24.7708,25.2127,25.7498,26.2869,26.5145,26.4326,26.3507,26.4831,26.7584,26.986,27.2136,27.346,27.4545,27.4917,27.5526,27.7802,28.1269,28.9021,29.7963,30.5001,30.8229,31.0981,31.3733,31.3629,31.162,30.961,30.8553,30.7496,30.4535,30.2526,31.0039,32.3743,34.078,35.6864,37.152,38.4748,39.7499,41.1203,43.062,44.6705,46.1837,46.9351,47.2579,47.5331,48.2845,49.4644,50.8348,52.2052,52.6708,52.5175,51.7928,50.9729,50.4387,50.4758,50.6558,50.1692,49.5874,48.8151,47.7571,46.461,45.1173,44.5355,43.668,42.8004,42.5043,42.7795, \
	16.114,16.4845,16.8073,17.0349,17.2625,17.4663,17.7892,18.2548,18.7443,19.2814,19.8185,20.2128,20.6071,21.0013,21.3956,21.8375,22.327,22.8165,23.3536,23.8669,24.3802,24.8697,25.2639,25.6344,26.1239,26.4467,26.3886,26.1877,26.1534,26.2858,26.561,26.7887,27.0401,27.2439,27.3048,27.3182,27.3553,27.5115,27.882,28.2762,28.9562,29.2314,29.2686,29.4009,29.6762,29.8085,29.7028,29.5495,29.3962,29.2905,29.2325,29.4125,30.878,32.6293,34.4282,36.3224,37.9308,39.444,40.9572,42.4228,43.7932,44.7827,45.2007,45.4283,45.7512,46.3121,47.3015,48.7195,49.8994,51.2222,51.307,50.8203,50.0957,49.2281,48.551,48.9691,49.3872,48.7101,48.1283,47.356,46.3932,45.24,43.8963,43.1716,42.685,41.9127,41.7118,41.987, \
	16.0595,16.4062,16.6576,16.8376,17.089,17.3404,17.6156,17.9861,18.4756,19.1079,19.7165,20.2298,20.6954,21.1849,21.6744,22.1639,22.6534,23.1429,23.7038,24.2885,24.8495,25.339,25.757,26.0799,26.4503,26.4875,26.2865,26.0618,25.9799,26.0647,26.2923,26.4961,26.8428,27.0466,27.1075,27.097,27.1104,27.2666,27.5894,27.9599,28.2351,28.2246,28.1904,28.2989,28.5027,28.6113,28.458,28.4952,28.699,28.9028,29.2256,30.1436,31.6092,33.0272,34.7785,36.6726,38.3287,40.0324,41.6884,43.1064,43.8578,44.1806,44.0273,43.9216,44.2921,45.0434,46.3186,47.8794,49.1545,50.2868,50.1335,49.5041,48.6842,47.7214,46.8539,47.1291,47.8328,47.0129,46.3359,45.6588,44.6484,43.5428,42.4372,41.5697,41.5116,40.9297,40.824,40.8612, \
	15.9812,16.3279,16.5555,16.7355,16.9393,17.1907,17.466,17.8364,18.3021,18.9582,19.6382,20.2229,20.7838,21.3447,21.9295,22.4904,22.9799,23.4694,24.0779,24.6626,25.2473,25.7844,26.2025,26.4301,26.5625,26.5044,26.2321,25.9121,25.7112,25.6769,25.8569,26.2512,26.574,26.7302,26.7912,26.7331,26.675,26.8074,27.1778,27.5007,27.6093,27.3131,27.4217,27.5779,27.7817,27.8426,27.7607,27.8931,28.454,28.9673,29.8377,31.2081,32.388,33.5679,35.1764,36.9277,38.6313,40.5731,42.2768,43.1234,43.1605,42.9596,42.6634,42.6054,43.071,43.9653,45.2404,46.9441,48.5525,49.3991,49.103,48.3307,47.3203,46.3099,45.2519,45.1462,45.85,45.1253,44.4958,43.8664,42.856,41.5123,40.74,40.0629,39.8144,39.3278,39.4125,39.6878, \
	15.7363,16.1544,16.382,16.5858,16.7896,17.0172,17.3639,17.8295,18.3428,18.9276,19.5837,20.2399,20.8722,21.5045,22.1607,22.7454,23.2825,23.8434,24.5234,25.0605,25.55,26.0157,26.3623,26.5185,26.6509,26.6166,26.249,25.8815,25.6329,25.6224,25.8024,26.0301,26.2101,26.3186,26.3796,26.2739,26.1444,26.2053,26.552,26.7796,26.6739,26.2587,26.6292,26.9282,27.2034,27.5025,27.8253,28.1958,28.8281,29.6747,30.9022,32.225,33.4049,34.4896,35.9075,37.4208,39.0768,40.971,42.0556,42.1404,41.7966,41.4052,41.2995,41.4795,42.0881,43.1251,44.4003,46.104,47.9981,48.559,48.0724,47.1096,45.9088,44.8508,43.9356,43.4966,43.8671,43.5709,42.8463,42.074,41.1112,40.0056,39.0904,38.6514,38.5934,38.4401,38.7629,39.7523, \
	15.3724,15.6952,15.9704,16.2695,16.5685,16.8913,17.3332,17.8703,18.455,19.0159,19.6245,20.3044,21.032,21.6643,22.2967,22.9528,23.5375,24.0985,24.6832,25.1489,25.5431,25.8898,26.1412,26.3688,26.5012,26.2764,25.9327,25.6127,25.4356,25.4251,25.5337,25.6899,25.7509,25.788,25.849,25.7433,25.59,25.6033,25.8071,25.9157,25.9766,26.1566,26.4319,26.7547,27.0775,27.5432,28.1041,28.7603,29.6069,30.8344,32.1096,33.3371,34.6122,35.6969,36.8292,38.1043,39.7604,40.9403,41.1679,40.5861,40.1471,39.9461,40.0309,40.4014,41.3432,42.4755,43.8935,45.5019,47.3484,47.4808,46.9466,45.841,44.5925,43.5345,42.7146,42.0852,42.408,42.0642,41.53,40.8529,39.9854,38.8322,37.7742,37.3352,37.6104,38.1237,39.1131,40.4835, \
	14.9846,15.2598,15.5112,15.8341,16.2521,16.6702,17.2311,17.8873,18.5434,19.1757,19.8081,20.4642,21.1442,21.7289,22.3374,22.9698,23.6021,24.1154,24.5573,24.9515,25.2744,25.5258,25.7296,25.862,25.9468,25.7934,25.4973,25.2488,25.0954,25.085,25.1697,25.2783,25.2679,25.2336,25.2945,25.1888,25.0117,24.9775,25.1336,25.3136,25.8031,26.364,26.7821,27.0811,27.404,27.9173,28.5734,29.4438,30.6238,31.9465,33.3169,34.6396,35.8672,36.8566,37.7984,38.9307,40.3963,40.2906,39.804,39.1269,38.6403,38.5346,38.8099,39.6089,40.7888,42.0639,43.5295,44.9951,45.9845,46.2122,45.8684,44.858,43.4667,42.3135,41.446,40.7689,40.806,40.748,40.309,39.6319,38.6691,37.5635,36.6008,36.6855,37.3893,38.5692,39.9396,41.3099, \
	14.8825,15.0863,15.2425,15.5653,16.0548,16.711,17.3433,17.9994,18.6318,19.3117,19.9679,20.6002,21.2564,21.8649,22.4496,22.9867,23.5714,23.9657,24.2647,24.5638,24.8152,25.019,25.1752,25.2838,25.3209,25.239,25.0381,24.8372,24.7077,24.6496,24.7105,24.7953,24.7134,24.6792,24.7639,24.6582,24.5049,24.4706,24.8173,25.3544,26.3438,27.0476,27.5133,27.7885,28.0637,28.5056,29.3284,30.5083,31.6882,33.0586,34.429,35.7993,37.0269,37.7782,38.482,39.1858,39.5086,38.9744,38.3449,37.6678,37.2765,37.266,37.7317,38.9116,40.282,41.7476,43.0703,44.0121,44.4778,44.753,44.695,43.8751,42.4361,41.1401,40.2249,39.8335,39.2041,39.2412,39.0403,38.5537,37.6385,36.4377,35.6178,36.0835,37.2634,38.9194,40.4803,41.8983, \
	14.828,15.0318,15.2594,15.6299,16.2384,16.8946,17.5031,18.1116,18.7678,19.4239,20.0563,20.6648,21.2733,21.8818,22.419,22.8846,23.3265,23.6017,23.8293,24.0808,24.3084,24.4408,24.5731,24.6817,24.7189,24.637,24.5551,24.4256,24.2723,24.1904,24.2037,24.1933,24.0876,24.0771,24.2095,24.1514,24.1171,24.2257,24.9771,25.9427,26.9798,27.9454,28.5064,28.5911,28.8187,29.3558,30.4881,31.7633,32.8003,33.8374,34.8268,35.7211,36.6629,37.6523,38.4037,38.5837,38.3352,37.6581,36.8858,36.2563,35.9602,36.1402,37.2249,38.5952,40.0132,41.336,42.4206,43.0292,43.3044,43.4368,43.3787,42.7016,41.5008,40.2047,39.1944,38.6601,37.983,37.7821,37.6764,37.285,36.5604,35.8357,35.4919,35.91,37.1851,39.0316,40.7353,42.2961, \
	14.7974,15.0488,15.3716,15.8611,16.4935,17.0782,17.6391,18.2238,18.8323,19.4885,20.0732,20.5865,21.1236,21.6845,22.174,22.5683,22.8911,23.1187,23.3225,23.574,23.7778,23.8863,23.9711,24.0559,24.1168,24.1064,24.0959,23.9902,23.7893,23.6122,23.5541,23.496,23.2237,23.356,23.6075,23.8113,24.0865,24.576,25.375,26.412,27.6157,28.4623,29.7375,29.8699,29.9546,29.8966,30.7908,31.7802,32.3888,32.9973,33.701,34.4524,35.3942,36.5742,37.5636,37.6008,37.0189,36.2942,35.5219,34.9877,34.8344,35.3953,36.8609,38.3741,39.6016,40.5911,41.3425,41.7129,41.8453,42.0253,42.0148,41.5758,40.7559,39.5075,38.2114,37.4867,37.0477,36.4182,36.3125,35.9212,35.3869,35.0908,35.1755,35.8317,37.2021,38.9057,40.7046,42.4083, \
	14.7905,15.0657,15.4838,16.0209,16.6056,17.1427,17.6799,18.2408,18.8255,19.4578,19.9949,20.413,20.8311,21.2729,21.691,22.0853,22.3605,22.5881,22.8157,23.0433,23.2471,23.3081,23.3691,23.4776,23.5624,23.5995,23.5653,23.412,23.1396,22.9149,22.8092,22.6083,22.4788,22.8016,23.1959,23.5187,24.032,24.7596,25.7014,26.8813,27.7279,28.265,28.8498,29.3154,29.4002,29.3421,29.7126,30.4163,30.882,31.6334,32.3848,33.2314,34.2684,35.4007,36.4854,36.4273,35.7503,34.978,34.3009,33.8619,33.9943,34.9361,36.4969,37.8673,38.8568,39.5129,39.9786,40.3014,40.4338,40.6138,40.6509,40.4976,39.9634,38.9054,37.4665,36.4561,36.0647,35.2924,34.9963,34.6049,34.3564,34.3935,34.9068,35.8963,37.3142,38.8751,40.674,42.4253, \
	14.7598,15.0589,15.5246,16.0855,16.6226,17.1359,17.6492,18.1863,18.7234,19.2605,19.7976,20.1681,20.4433,20.7899,21.1604,21.507,21.8061,22.0813,22.3089,22.4651,22.5975,22.6823,22.767,22.8994,22.9842,22.9975,22.9394,22.8099,22.6804,22.5509,22.3976,22.4348,22.6148,23.0566,23.2842,23.7499,24.3585,25.1098,26.0517,27.0173,27.5544,27.9249,28.2715,28.4991,28.5363,28.5496,28.682,29.0048,29.4229,30.1743,30.9733,31.9151,33.0474,34.3702,35.4548,35.3491,34.5292,33.7569,33.1275,32.8789,33.3446,34.5721,35.9425,36.932,37.6833,38.149,38.5195,38.9375,39.2128,39.3451,39.3823,39.3242,38.9804,38.1605,36.8645,35.616,34.8437,34.3095,33.68,33.7172,33.9924,34.4105,35.1142,36.3418,37.7121,39.2253,40.8338,42.5374, \
	14.7054,15.0282,15.4939,16.0786,16.6157,17.0576,17.4995,17.9652,18.4547,18.9204,19.3384,19.7089,20.0079,20.2831,20.6298,20.9764,21.2517,21.4793,21.6593,21.8154,21.9716,22.104,22.2364,22.3212,22.4059,22.4193,22.3612,22.3031,22.2212,22.0441,21.867,22.3803,22.965,23.4783,23.8726,24.3383,24.9706,25.6744,26.3305,26.8914,27.1428,27.3704,27.6695,27.8495,27.9104,27.9238,28.0562,28.3314,28.7256,29.1437,29.7999,30.8369,32.1121,33.4348,34.329,34.1281,33.3558,32.5359,31.954,31.9912,32.695,34.0177,35.1024,35.8061,36.3671,36.7375,37.108,37.5737,38.0393,38.2193,38.1613,38.1032,37.9023,37.2252,36.072,34.7759,33.5751,32.898,32.8875,33.496,34.3427,35.1893,36.1311,37.311,38.4909,39.7184,40.9936,42.5068, \
	14.5795,14.9261,15.3442,15.9051,16.4898,16.9079,17.2784,17.6726,18.0669,18.485,18.8316,19.1068,19.4773,19.7763,20.0992,20.3982,20.6258,20.8534,21.0334,21.2134,21.4172,21.5972,21.7058,21.7905,21.8515,21.8648,21.783,21.7487,21.7858,21.823,22.0506,22.6353,23.2677,23.8524,24.3657,24.879,25.4399,25.977,26.3475,26.5989,26.6598,26.7684,26.996,27.1998,27.3084,27.3455,27.4541,27.7055,28.1474,28.6131,29.1502,30.0444,31.3672,32.4519,33.0128,32.8118,32.1348,31.3625,30.9235,31.1987,32.1405,33.13,34.0242,34.6327,35.0984,35.5165,35.9346,36.4479,36.9612,37.0459,36.7974,36.8345,36.6336,35.9565,34.9462,33.8882,32.6873,31.8674,32.8092,34.0844,35.2167,36.2537,37.3384,38.4231,39.6506,40.8305,42.0104,43.238, \
	14.406,14.705,15.0755,15.5173,16.0783,16.5678,16.9858,17.3325,17.6553,18.002,18.3486,18.6714,18.9705,19.2219,19.4971,19.7485,20.0238,20.2514,20.4314,20.6352,20.8628,21.0666,21.1752,21.2123,21.2733,21.3104,21.2523,21.2181,21.4933,21.9114,22.3771,22.938,23.5465,24.0836,24.5969,25.1102,25.5521,25.9463,26.1978,26.2587,26.1768,26.1902,26.394,26.5978,26.754,26.8625,26.9235,27.1273,27.6406,28.1539,28.7624,29.4186,30.4794,31.3737,31.6965,31.4956,30.8661,30.1414,30.0357,30.3586,31.2052,32.0042,32.8032,33.3641,33.8298,34.2478,34.6659,35.1316,35.4544,35.4916,35.4335,35.6135,35.603,34.926,33.8204,32.6195,31.8472,31.9796,33.1595,34.5299,35.805,36.9373,38.1172,39.2971,40.4294,41.5617,42.8368,43.9215, \
	14.2801,14.5791,14.902,15.2486,15.7143,16.18,16.5742,16.8971,17.1723,17.4951,17.8656,18.2361,18.4875,18.7151,18.9427,19.1703,19.4455,19.6731,19.8769,20.0569,20.3322,20.5598,20.5969,20.5865,20.5998,20.8274,20.9122,20.9731,21.4864,22.1426,22.7035,23.2168,23.7063,24.172,24.59,24.9843,25.3786,25.7252,25.9766,26.0138,25.8129,25.7548,25.9348,26.1624,26.3424,26.4272,26.4881,26.6681,27.1338,27.7899,28.4223,28.9832,29.4965,30.105,30.3802,30.2269,29.7403,29.5394,29.6242,29.9232,30.2222,30.6879,31.4869,32.0954,32.5611,32.9792,33.3496,33.7201,33.9953,34.0801,34.0696,34.3925,34.4296,33.8002,32.6469,31.5413,31.3642,32.0918,33.4145,34.8325,36.0124,37.1447,38.3246,39.5998,40.6844,41.8643,43.0919,44.0813, \
	14.1304,14.3818,14.6808,14.9561,15.3741,15.7684,16.0912,16.3665,16.6417,16.9169,17.3112,17.6816,17.9807,18.2321,18.5073,18.7825,18.9863,19.2378,19.4654,19.6454,19.9206,20.1244,20.1615,20.1273,20.1406,20.5111,20.8577,21.2758,21.8129,22.3262,22.8395,23.2576,23.628,24.0461,24.4165,24.787,25.1098,25.3851,25.5889,25.6974,25.6632,25.5575,25.6899,25.9175,26.0975,26.1346,26.148,26.2565,26.6032,27.2355,27.8917,28.4526,28.9183,29.2173,29.4211,29.4106,29.1383,29.1516,29.4507,29.7259,29.8583,30.0621,30.4325,30.8744,31.2925,31.7105,31.9858,32.261,32.4886,32.7162,32.801,33.1714,33.3514,32.8648,31.7592,31.1297,31.1431,32.0611,33.6696,34.9923,36.0294,37.0664,38.1035,39.2834,40.4633,41.6432,42.7755,43.6221, \
	13.9569,14.1607,14.4121,14.6873,15.034,15.3568,15.6082,15.8835,16.2301,16.5529,16.8758,17.2701,17.6167,18.011,18.4052,18.7043,18.8366,19.0404,19.2681,19.5433,19.7947,19.9509,20.0356,20.0966,20.2528,20.5518,20.9461,21.3642,21.7822,22.2241,22.5708,22.8222,23.2879,23.8012,24.1954,24.5421,24.8411,25.0687,25.2249,25.3335,25.323,25.1935,25.3021,25.5059,25.6621,25.723,25.7602,25.8687,26.2392,26.7763,27.3134,27.8743,28.2924,28.5438,28.7,28.7372,28.56,28.8115,29.1819,29.4095,29.4943,29.6267,29.8305,30.0819,30.3333,30.5609,30.6933,30.8019,30.9819,31.3047,31.5799,31.9028,32.3208,32.1199,31.3476,30.8372,30.9696,31.84,33.5437,34.7712,35.8083,36.7977,37.7395,38.7766,39.9565,41.0888,42.0783,43.0201, \
	13.831,14.011,14.191,14.4662,14.7414,14.969,15.2919,15.6861,16.1756,16.6651,16.988,17.3108,17.7289,18.2184,18.6603,18.8641,18.925,19.0098,19.2136,19.5126,19.8116,19.9916,20.0526,20.1136,20.2221,20.545,20.8678,21.0478,21.2516,21.5983,21.9449,22.2916,22.781,23.1991,23.7838,24.2019,24.5009,24.7047,24.8609,24.8981,24.84,24.7105,24.8667,25.0467,25.1791,25.24,25.301,25.5286,25.9467,26.3647,26.8066,27.2009,27.5237,27.7513,27.9075,27.9685,28.1008,28.5903,28.9608,29.0694,29.1065,29.1675,29.2999,29.456,29.6122,29.7922,29.9008,29.9856,30.1894,30.5122,30.7636,30.9912,31.3855,31.5655,31.1265,30.6399,30.7723,31.6189,33.0845,34.2644,35.3014,36.3385,37.328,38.2698,39.5449,40.582,41.5238,42.418, \
	13.7765,13.9327,14.1365,14.3641,14.6393,14.9146,15.3088,15.8698,16.4069,16.7535,17.0525,17.423,17.7696,18.2115,18.6296,18.8334,18.8467,18.8601,19.0163,19.2677,19.5667,19.8658,19.9029,19.8448,19.9534,20.3477,20.361,20.422,20.5543,20.8058,21.1048,21.4276,21.9171,22.478,23.158,23.7189,24.0417,24.2455,24.3779,24.4389,24.3808,24.2513,24.4075,24.5637,24.6961,24.7808,24.8656,25.2122,25.6303,26.0008,26.3236,26.5512,26.755,26.9588,27.1864,27.3664,27.9988,28.5597,28.7635,28.8006,28.7426,28.7321,28.8169,28.8064,28.8197,28.9521,29.0607,29.1693,29.4207,29.8149,30.114,30.5082,31.0215,31.2015,30.834,30.395,30.5511,31.3739,32.5777,33.7576,34.7946,35.8793,36.9164,37.8582,38.8,39.9799,40.8266,41.6255, \
	13.7459,13.9259,14.1297,14.3573,14.6325,15.0506,15.5639,16.0057,16.3286,16.6276,17.0219,17.4399,17.7152,18.038,18.337,18.4932,18.5304,18.6152,18.819,19.0704,19.298,19.478,19.5627,19.5761,19.7561,19.9361,19.9256,19.9866,20.119,20.2513,20.4313,20.7066,21.1008,21.5665,22.2226,22.7359,23.0826,23.3578,23.6568,23.9083,23.9692,23.8635,23.9721,24.1283,24.2845,24.3216,24.4302,24.8245,25.2187,25.5416,25.8168,25.973,26.1768,26.4996,26.8463,27.2167,28.1586,28.4576,28.5424,28.5557,28.4738,28.3681,28.31,28.1329,28.0272,28.0644,28.1491,28.3291,28.652,29.1653,29.631,30.1443,30.5861,30.7899,30.4938,30.0548,30.33,31.2242,32.2613,33.2984,34.1926,35.2773,36.3619,37.3514,38.2932,39.3303,40.2721,41.1663, \
	13.7866,13.9428,14.1228,14.3266,14.578,14.9247,15.3666,15.737,16.0598,16.4541,16.8722,17.3141,17.6369,17.8883,18.0921,18.2483,18.3331,18.4893,18.7169,18.9207,19.0531,19.114,19.1512,19.2359,19.4397,19.5483,19.5616,19.6464,19.8026,19.9112,20.0436,20.2712,20.5702,20.7978,21.1921,21.5387,21.8377,22.2082,22.6739,23.1396,23.4148,23.4758,23.5129,23.6691,23.8491,23.8624,23.9472,24.3653,24.7119,24.9633,25.2862,25.5138,25.908,26.469,26.9823,27.4003,27.866,28.1174,28.226,28.2155,28.086,27.9565,27.9223,27.8166,27.6871,27.7004,27.809,28.0128,28.3356,28.8013,29.3146,29.8279,30.246,30.3069,29.9394,29.6432,30.0375,31.0745,31.7783,32.5059,33.5429,34.7705,36.0932,37.2731,38.215,39.3949,40.1939,40.9452, \
	13.7798,13.936,14.116,14.3198,14.5236,14.775,15.074,15.4207,15.8149,16.2568,16.6987,17.1406,17.4634,17.7148,17.8948,18.051,18.2072,18.3872,18.6148,18.7948,18.8319,18.8453,18.8586,18.991,19.1472,19.232,19.2691,19.3777,19.5339,19.6662,19.8224,20.0262,20.2538,20.3624,20.4472,20.651,20.8786,21.2014,21.6195,22.0852,22.5271,22.8261,22.8871,23.1861,23.3661,23.427,23.607,24.0251,24.1099,24.4803,25.0651,25.6736,26.2583,26.724,27.0944,27.3697,27.5497,27.7297,27.8144,27.8278,27.7459,27.6402,27.6297,27.5955,27.585,27.6221,27.6831,27.8393,28.1145,28.4374,28.9268,29.4401,29.8582,29.8001,29.5278,29.5411,30.2211,30.9487,31.4143,31.9514,32.8933,34.0256,35.3959,36.7663,37.8986,39.1737,39.8299,40.2956, \
	13.7729,13.9053,14.0853,14.2653,14.4215,14.6491,14.9005,15.2471,15.6652,16.1071,16.5252,16.9194,17.2423,17.4461,17.6261,17.8061,17.9384,18.0946,18.3699,18.5499,18.5632,18.6242,18.7327,18.8413,18.9499,18.987,19.0718,19.2042,19.3604,19.4927,19.6013,19.7575,19.9613,20.0699,20.1546,20.3108,20.5384,20.7422,20.9699,21.2689,21.5917,21.8431,22.2374,22.7031,22.9783,23.0869,23.2193,23.5421,24.0792,24.6639,25.201,25.7143,26.18,26.5981,26.9209,27.1724,27.3285,27.4371,27.4981,27.5352,27.501,27.4191,27.361,27.3505,27.3877,27.4724,27.5572,27.6658,27.822,28.0972,28.4676,28.8619,29.1848,29.3171,29.1876,29.3914,30.0952,30.5847,30.9313,31.5161,32.1246,33.1855,34.3654,35.831,37.2013,38.286,38.8469,39.2174, \
	13.7423,13.8746,14.0308,14.187,14.3194,14.5232,14.8222,15.1689,15.5393,15.9574,16.3517,16.7221,17.0211,17.1535,17.1907,17.2516,17.3602,17.5402,17.8869,18.0907,18.223,18.3792,18.5116,18.5964,18.6811,18.7897,18.8745,19.0545,19.2107,19.3192,19.4278,19.5602,19.6926,19.8011,19.9573,20.1373,20.3173,20.4735,20.6773,20.9049,21.0849,21.3601,21.7544,22.1487,22.4477,22.6753,23.022,23.5591,24.0724,24.538,25.0752,25.5646,26.0065,26.3532,26.6284,26.8322,26.9646,27.0732,27.1341,27.1713,27.1608,27.1265,27.1161,27.1056,27.1666,27.2751,27.4075,27.5161,27.6485,27.8285,28.0085,28.2361,28.4161,28.6675,28.7999,29.0989,29.6836,30.1017,30.4483,31.0092,31.6892,32.2977,33.3824,34.6575,36.2184,36.9697,37.5783,38.1868, \
	13.7592,13.8678,14.0002,14.1325,14.2649,14.4449,14.6963,14.9716,15.3182,15.7125,16.1543,16.5724,16.8714,16.7896,16.5172,16.3639,16.3772,16.7239,17.261,17.5838,18.0257,18.2771,18.3619,18.4467,18.5314,18.6162,18.701,18.8333,18.9657,19.1219,19.2305,19.3391,19.4714,19.6276,19.76,19.94,20.0962,20.2286,20.3371,20.4933,20.6971,21.0676,21.438,21.6895,21.9647,22.359,22.8723,23.3141,23.7084,24.055,24.5207,25.034,25.4521,25.8226,26.0978,26.254,26.434,26.5425,26.5321,26.6644,26.773,26.834,26.8949,26.9083,26.9692,27.0778,27.2578,27.4616,27.594,27.6788,27.6683,27.7293,27.814,28.1369,28.5311,28.9016,29.2958,29.7139,30.132,30.7405,31.4205,32.029,32.828,33.9127,35.1878,36.0344,36.7858,37.9657, \
	13.7524,13.8847,13.9695,14.0781,14.2105,14.3428,14.499,14.7028,15.0257,15.4913,16.0046,16.4227,16.5313,16.4018,16.2009,16.0714,16.0609,16.4552,16.9685,17.3627,17.757,18.1036,18.2122,18.3208,18.4294,18.5141,18.6227,18.7075,18.8636,18.9722,19.057,19.1656,19.2979,19.4779,19.6103,19.7427,19.8751,19.9836,20.0922,20.2484,20.5236,20.8703,21.1693,21.3493,21.6007,21.9712,22.3654,22.6645,22.9635,23.1911,23.7282,24.2891,24.7072,24.9824,25.2577,25.4853,25.7367,25.9405,26.1205,26.3481,26.5043,26.6129,26.6738,26.711,26.7719,26.8805,27.1319,27.4548,27.6348,27.5767,27.4234,27.3415,27.6405,28.1538,28.6433,29.0614,29.3842,29.7785,30.3394,30.9479,31.5326,32.165,33.1306,34.1677,35.3,36.6227,38.374,39.3159, \
	13.7217,13.8541,13.9865,14.095,14.2274,14.2884,14.4207,14.5531,14.7569,15.1274,15.5693,15.8683,16.0721,16.0378,15.9797,15.9931,16.1731,16.4959,16.8426,17.0702,17.2026,17.6206,17.872,18.0282,18.1606,18.3168,18.4492,18.5578,18.6901,18.8225,18.9073,18.9921,19.1244,19.2806,19.413,19.5454,19.6778,19.7863,19.8949,20.0749,20.3739,20.673,20.8768,21.0568,21.3082,21.5596,21.7634,21.9672,22.2424,22.4224,23.0786,23.6395,23.9623,24.1423,24.3699,24.7404,25.0394,25.4813,26.1136,26.3651,26.4736,26.487,26.4527,26.4898,26.5508,26.707,26.9584,27.2098,27.2946,27.2603,27.2261,27.3346,27.9193,28.4326,28.9698,29.4592,29.9249,30.3906,30.9753,31.6315,32.3591,33.3961,34.4332,35.5179,36.5073,37.7824,39.2957,40.4756, \
	13.691,13.8472,14.0034,14.1358,14.1967,14.3053,14.4139,14.5225,14.7025,14.9539,15.2529,15.5043,15.7081,15.8405,15.9253,16.0577,16.2853,16.5843,16.8119,16.9443,16.9338,17.209,17.6271,17.8785,18.0347,18.1671,18.2757,18.3843,18.5166,18.6252,18.71,18.8185,18.9271,19.0595,19.1919,19.3243,19.4804,19.6128,19.7452,19.9014,20.2004,20.4995,20.6556,20.8356,21.0633,21.2671,21.4232,21.5794,21.807,22.1537,22.7622,23.1565,23.4317,23.6593,23.9584,24.3764,24.8421,25.7363,26.2496,26.3344,26.3953,26.3849,26.2316,26.0068,26.0678,26.4144,26.404,26.3935,26.5021,26.8487,27.2192,27.7563,28.341,28.8305,29.2962,29.7619,30.2752,30.8123,31.5875,32.4817,33.5664,34.9367,36.2119,37.2489,37.9527,39.0374,40.3601,41.5877, \
	13.6842,13.8166,13.9965,14.1289,14.2137,14.2985,14.3832,14.4918,14.6718,14.8756,15.0794,15.3308,15.5822,15.8098,16.0613,16.2889,16.5165,16.7203,16.8765,16.9612,16.9746,17.1308,17.5488,17.8717,18.0041,18.0888,18.1498,18.2822,18.3907,18.4755,18.5365,18.6212,18.7298,18.8622,18.9708,19.0793,19.2355,19.3917,19.6193,19.8469,20.0507,20.3498,20.6488,20.805,20.9374,21.0697,21.2259,21.4059,21.5859,21.885,22.3268,22.7687,23.1154,23.462,23.9515,24.6314,25.2876,25.8961,26.1952,26.1371,26.0314,25.8543,25.6057,25.3334,25.5848,25.9076,26.2305,26.5771,26.9952,27.5085,27.7123,27.8685,28.358,28.8713,29.3846,29.8979,30.435,31.0673,31.8425,32.9748,34.2499,35.6679,37.0859,38.1706,39.0648,40.1495,41.2818,42.4617, \
	13.6773,13.8097,13.9421,14.0745,14.183,14.2678,14.4002,14.5088,14.6411,14.8211,15.0249,15.3002,15.5992,15.922,16.3163,16.5677,16.7001,16.8325,16.9172,16.9782,17.0392,17.2191,17.5182,17.7934,17.9734,18.0344,18.1191,18.2277,18.3125,18.3734,18.4344,18.5191,18.6277,18.7839,18.8449,18.9296,19.0144,19.1944,19.541,19.8401,19.9963,20.2239,20.4991,20.6553,20.7877,20.8962,21.0762,21.3038,21.4838,21.7114,22.0343,22.4285,23.0133,23.7408,24.4684,25.0293,25.495,25.7464,25.8312,25.7255,25.5246,25.276,25.0275,24.9456,25.5779,25.996,26.5331,27.0464,27.5359,27.8111,27.8721,28.0045,28.4702,28.9358,29.4491,29.9863,30.5472,31.1557,31.8833,32.8965,34.2193,35.6373,37.1029,38.4256,39.5579,40.6902,41.7749,42.8119, \
	13.6467,13.8028,13.9114,14.02,14.1524,14.2848,14.4171,14.5257,14.6581,14.8143,15.0419,15.3171,15.6638,16.0818,16.5237,16.7037,16.7885,16.8732,16.958,16.9951,17.1037,17.2361,17.4399,17.6437,17.8237,17.9323,18.017,18.1018,18.1866,18.2713,18.3323,18.4409,18.5732,18.7056,18.838,18.9228,19.0313,19.1875,19.4151,19.7142,19.9894,20.217,20.3732,20.4818,20.6142,20.7465,20.9265,21.1541,21.3818,21.5617,21.837,22.2789,23.0302,23.8768,24.5568,24.9987,25.3691,25.4777,25.4196,25.1473,24.9225,24.8406,24.9254,25.3197,26.0949,26.5129,27.0024,27.4443,27.7433,27.8757,27.9367,28.1643,28.7252,29.1909,29.6328,30.0746,30.5879,31.1965,31.8526,32.723,33.8553,35.2257,36.6913,38.1093,39.3844,40.6119,41.8394,42.7813, \
	13.6398,13.7722,13.9046,14.0131,14.1455,14.2779,14.4103,14.5427,14.6988,14.855,15.0826,15.3817,15.7521,16.075,16.374,16.5778,16.734,16.8664,16.9749,17.0597,17.0969,17.1816,17.2902,17.4702,17.6264,17.7826,17.8673,17.9759,18.0369,18.0978,18.1588,18.2674,18.4474,18.6274,18.8073,18.9635,19.0721,19.1569,19.3131,19.5883,19.9349,20.1387,20.2235,20.3083,20.4406,20.573,20.7768,21.0521,21.2797,21.4359,21.7111,22.1768,22.9996,23.8224,24.4309,24.849,25.148,25.2328,24.9842,24.4976,24.4395,24.7624,25.2519,25.9318,26.5165,26.9584,27.3289,27.6279,27.7841,27.8927,27.9298,28.2288,28.9088,29.4935,29.9116,30.2582,30.7001,31.3086,31.8934,32.74,33.7294,34.9093,36.2321,37.5548,38.8776,40.2003,41.5707,42.6078, \
	13.6329,13.7653,13.8977,14.0301,14.1387,14.2949,14.4034,14.5596,14.7158,14.8958,15.1234,15.4462,15.7929,16.0443,16.2243,16.4043,16.5843,16.7643,16.8967,17.0529,17.1138,17.1986,17.3072,17.4157,17.5481,17.6567,17.7653,17.8262,17.911,17.9719,18.0329,18.1177,18.2977,18.4777,18.7053,18.9091,19.0653,19.1738,19.2824,19.4862,19.69,19.8938,20.05,20.1824,20.3386,20.4948,20.7462,21.069,21.2966,21.4052,21.6566,22.0985,22.8022,23.5536,24.1383,24.5802,24.8554,24.9402,24.525,24.0384,24.266,24.9222,25.6973,26.3535,26.8192,27.1658,27.4887,27.6925,27.7534,27.743,27.8039,28.0791,28.8067,29.6057,30.1428,30.5371,30.9789,31.4922,32.0532,33.1378,33.9844,34.9263,36.011,37.3337,38.6565,39.9316,41.3496,42.5295, \
	13.6499,13.7823,13.9147,14.047,14.1556,14.288,14.4442,14.5766,14.7328,14.9127,15.1642,15.4632,15.786,15.9898,16.0508,16.207,16.3632,16.5432,16.7232,16.9032,17.0355,17.1679,17.2765,17.3613,17.4936,17.6022,17.7108,17.7479,17.8327,17.9175,17.9784,18.087,18.1956,18.3756,18.5794,18.807,19.0108,19.1432,19.2517,19.3603,19.4927,19.6965,19.9479,20.1517,20.3079,20.4879,20.7393,21.0622,21.2898,21.4221,21.6498,21.9964,22.5335,23.142,23.703,24.1686,24.4439,24.5048,24.2563,24.103,24.4496,25.1296,25.9048,26.5847,27.0742,27.3732,27.6008,27.7332,27.6751,27.4266,27.3209,27.6914,28.4189,29.1703,29.8741,30.4826,30.9959,31.5092,32.0939,33.0596,34.049,35.0861,36.1708,37.3983,38.6258,39.7581,40.8428,41.7846, \
	13.6669,13.7992,13.9316,14.064,14.1726,14.3049,14.4611,14.5935,14.7497,14.9297,15.1335,15.3849,15.6601,15.9116,16.0201,16.1287,16.2611,16.3697,16.5259,16.7297,16.8858,17.042,17.1744,17.283,17.4154,17.5477,17.6563,17.6935,17.802,17.8868,17.9954,18.0801,18.1887,18.2973,18.4535,18.7287,18.9801,19.1601,19.2687,19.3535,19.462,19.642,19.9887,20.2163,20.3487,20.5049,20.6848,20.9125,21.1877,21.3677,21.5953,21.8705,22.241,22.7305,23.2438,23.6856,23.9847,23.998,23.8685,24.0247,24.4904,25.1703,25.8979,26.554,27.115,27.5092,27.6654,27.5835,27.3112,27.1103,27.076,27.3274,27.8169,28.4968,29.2482,29.9282,30.5367,31.1214,31.7776,32.648,33.8279,34.9602,35.9972,37.1295,38.1666,39.0132,39.955,40.9445};
	vPointsOut.reserve(105*88);
	for(int x=0;x<=104;++x)
	{
		for(int y=0;y<=87;++y)
		{
			vPointsOut.push_back(Point2(x,y,arr[x*88+y]));
		}
	}
}
